@@ -1,6 +1,6 @@
 beast123 <- function(Y, metadata=list(), prior=list(), mcmc=list(), extra=list(),
                      season = c('harmonic','svd','dummy','none'),
-					 method = c('bayes','bic', 'aic','aicc','hic'), ... )
+					 method = c('bayes','bic', 'aic','aicc','hic','bic0.25','bic0.5','bic1.5','bic2'), ... )
 {
   # tmplist = list(...)
   # if (!hasArg("Y") || is.list(Y) || length(Y)==1)  {
@@ -22,13 +22,9 @@ beast123 <- function(Y, metadata=list(), prior=list(), mcmc=list(), extra=list()
     			  metadata$startTime=tmp(1)
     			  metadata$deltaTime=tmp(2)
     			}         
-      } else if (is.list(metadata)) {
-        # do nothing 
-      }
-      else{
-        # this branch should be never touched!
-        metatata=list()
-      }
+      } 
+	  else if (is.list(metadata)) {} # do nothing 
+      else {  metadata=list()  }     # this branch should be never touched!  	  
  } else   {
       if( is.numeric(metadata) )    {
         tmp       = metadata
@@ -36,35 +32,25 @@ beast123 <- function(Y, metadata=list(), prior=list(), mcmc=list(), extra=list()
         if(length(tmp)==1){
           metadata$period=tmp(1)	
         } 		 
-      } else if (is.list(metadata)) {
-        # do nothing 
-      }  else{
-        # this branch should be never touched!
-        metadata=list()
-      }
+      } 
+	  else if (is.list(metadata)) {} # do nothing 
+      else {  metadata=list()  }     # this branch should be never touched!     
       
-}
+ }
 
-  if (is.null(metadata$season))
-    metadata$season=season;
+ if (is.null(metadata$season)) metadata$season=season
   
- method = match.arg(method) 
+ method =  match.arg(method) 
  funstr =  paste0("beast_",method)
  
-  if ( hasArg("cputype") )  {
-    cputype = list(...)[['cputype']]  
-    cputype = switch(cputype, sse=1, avx2=2, avx512=3);	
-	ANS     = .Call( BEASTV4_rexFunction, list(funstr,Y,metadata,prior,mcmc,extra,cputype),   212345)   		   
- } else {
-    if (hasArg("local")){ # run the local developer's version of Rbeast
+
+ if (hasArg("local")){ # run the local developer's version of Rbeast
 		#ANS  = .xxxCall( "rexFunction1", list(funstr,Y,metadata,prior,mcmc,extra),   212345, PACKAGE="Rbeast.mexw64")  
-	} else{
+ } else{
 	    ANS  = .Call( BEASTV4_rexFunction, list(funstr,Y,metadata,prior,mcmc,extra),   212345)   		   
-	}		   
- }
- 
- invisible(return(ANS))
-    
+ }		   
+
+ invisible(return(ANS))  
 }
 
 meanfilter <- function(x,n=5){filter(x,rep(1,n), sides=2)}
