@@ -1,6 +1,6 @@
 #include "abc_000_macro.h"
 #include "abc_000_warning.h"
-#if defined(WIN64_OS) 
+#if defined(OS_WIN64) 
 #include <stdio.h>   
 #include <io.h>      
 #include <math.h>
