@@ -1,6 +1,6 @@
 #pragma once
-#define R_INTERFACE   0
-#define M_INTERFACE   0
+#define R_INTERFACE    0
+#define M_INTERFACE    0
 #define P_INTERFACE    0
 #define MYMAT_LIBRARY   1
 #define MKL_LIBRARY     0
@@ -54,36 +54,30 @@
         #define MATLAB_DEFAULT_RELEASE  R2017b
 #endif
 #ifdef _MSC_VER
-	#define MSVC_COMPILER
+	#define COMPILER_MSVC
 #elif defined(__clang__)
-	#define CLANG_COMPILER
+	#define COMPILER_CLANG
 #elif (defined(__GNUC__)||defined(__GNUG__)) && !(defined(__clang__)||defined(__INTEL_COMPILER))
-	#define GCC_COMPILER
+	#define COMPILER_GCC
 #elif defined(__SUNPRO_C)||defined(__SUNPRO_CC)
-	#define SOLARIS_COMPILER
+	#define COMPILER_SOLARIS
 #endif
 #if defined(_WIN64)||defined (__MINGW64__) && defined(_WIN32) && !defined(__i386)  && !defined(__i686) && !defined(i386) && !defined(__i686)
-	#define WIN64_OS
+	#define OS_WIN64
 #elif defined(_WIN32) && !defined(_WIN64)
-	#define WIN32_OS
+	#define OS_WIN32
 #endif
 #if defined(__APPLE__ ) && defined (__MACH__) 
-	#define MAC_OS
+	#define OS_MAC
 #endif
 #if defined(__linux__) 
-	#define LINUX_OS
-    #ifndef _GNU_SOURCE
-		#define _GNU_SOURCE 
-    #endif
+	#define OS_LINUX
 #endif
 #if (defined(unix)||defined(__unix__)||defined(__unix) ) && !defined(__APPLE__)
-	#define UNIX_OS
-	#ifndef _GNU_SOURCE
-		#define _GNU_SOURCE 
-	#endif
+	#define OS_UNIX	
 #endif
 #if defined(sun) && defined(__sun) && defined(__SVR4) 
-	#define SOLARIS_OS
+	#define OS_SOLARIS
 #endif
 #if _WIN64||__amd64__||defined(__LP64__)||(defined(__x86_64__) &&    !defined(__ILP32__) )||defined(_M_X64)||defined(__ia64)||defined (_M_IA64)||defined(__aarch64__)||defined(__powerpc64__)
 	#define TARGET_64
@@ -97,14 +91,48 @@
 		#define TARGET_32
 	#endif
 #endif
-#if defined(__aarch64__)
-    #define  ARM64_OS
+#if defined(__x86_64__)||defined(_M_X64)
+	#define  cpu_x86_64
+#elif defined(i386)||defined(__i386__)||defined(__i386)||defined(_M_IX86)
+	#define  cpu_x86_32 
+#elif defined(__aarch64__)||defined(_M_ARM64)
+	#define   cpu_ARM64
+#elif defined(mips)||defined(__mips__)||defined(__mips)
+	#define   cpu_MIPS
+#elif defined(__sh__)
+	#define   cpu_SUPERH
+#elif defined(__powerpc)||defined(__powerpc__)||defined(__powerpc64__)||defined(__POWERPC__)||defined(__ppc__)||defined(__PPC__)||defined(_ARCH_PPC)
+	#define   cpu_POWERPC
+#elif defined(__PPC64__)||defined(__ppc64__)||defined(_ARCH_PPC64)
+	#define   cpu_POWERPC64
+#elif defined(__sparc__)||defined(__sparc)
+	#define   cpu_SPARC	
+#elif defined(__ARM_ARCH_2__)
+	#define   cpu_ARM2
+#elif defined(__ARM_ARCH_3__)||defined(__ARM_ARCH_3M__)
+	#define   cpu_ARM3
+#elif defined(__ARM_ARCH_4T__)||defined(__TARGET_ARM_4T)
+	#define   cpu_ARM4T
+#elif defined(__ARM_ARCH_5_)||defined(__ARM_ARCH_5E_)
+	#define   cpu_ARM5
+#elif defined(__ARM_ARCH_6T2_)||defined(__ARM_ARCH_6T2_)
+	#define   cpu_ARM6T2
+#elif defined(__ARM_ARCH_6__)||defined(__ARM_ARCH_6J__)||defined(__ARM_ARCH_6K__)||defined(__ARM_ARCH_6Z__)||defined(__ARM_ARCH_6ZK__)
+	#define   cpu_ARM6
+#elif defined(__ARM_ARCH_7__)||defined(__ARM_ARCH_7A__)||defined(__ARM_ARCH_7R__)||defined(__ARM_ARCH_7M__)||defined(__ARM_ARCH_7S__)
+	#define   cpu_ARM7
+#elif defined(__ARM_ARCH_7A__)||defined(__ARM_ARCH_7R__)||defined(__ARM_ARCH_7M__)||defined(__ARM_ARCH_7S__)
+	#define   cpu_ARM7A
+#elif defined(__ARM_ARCH_7R__)||defined(__ARM_ARCH_7M__)||defined(__ARM_ARCH_7S__)
+	#define   cpu_ARM7R
+#elif defined(__ARM_ARCH_7M__)
+	#define   cpu_ARM7M
+#elif defined(__ARM_ARCH_7S__)
+	#define   cpu_ARM7S
+#elif defined(__m68k__)
+	#define   cpu_M68K 
 #endif
-#if defined(__powerpc)||defined(__powerpc__)||defined(__powerpc64__)||defined(__POWERPC__)||defined(__ppc__)||defined(__PPC__)||defined(_ARCH_PPC) \
-     ||defined(__PPC64__)||defined(__ppc64__)||defined(_ARCH_PPC64)
-	#define POWERPC_OS
-#endif
-#if defined(TARGET_32) && defined (MSVC_COMPILER)
+#if defined(TARGET_32) && defined (COMPILER_MSVC)
 	#define _CRT_SECURE_NO_WARNINGS
 	#pragma warning (disable: 4703) 
 #endif
@@ -112,20 +140,20 @@
 		#define PCGRAND_LIBRARY 1
 		#define MKLRAND_LIBRARY 0
 #endif
-#if defined(MSVC_COMPILER)
+#if defined(COMPILER_MSVC)
 		#define INLINE    __inline
 		#define _restrict __restrict
         #define UNUSED_DECORATOR 
-#elif defined(SOLARIS_COMPILER) 
+#elif defined(COMPILER_SOLARIS) 
 		#define INLINE     inline 
 		#define _restrict _Restrict 
         #define UNUSED_DECORATOR 
-#elif defined(GCC_COMPILER)||defined(CLANG_COMPILER)
+#elif defined(COMPILER_GCC)||defined(COMPILER_CLANG)
 		#define INLINE     inline
 		#define _restrict __restrict__		
         #define UNUSED_DECORATOR  __attribute__((unused))
 #endif
-#if   defined(LINUX_OS) && ( defined(GCC_COMPILER)||defined(CLANG_COMPILER) )
+#if   defined(OS_LINUX) && ( defined(COMPILER_GCC)||defined(COMPILER_CLANG) )
 	#ifdef _GNU_SOURCE 
 		#include <features.h>
 		#ifndef __USE_GNU
@@ -140,7 +168,7 @@
 		#undef _GNU_SOURCE 
 	#endif
 #endif
-#ifdef MSVC_COMPILER
+#ifdef COMPILER_MSVC
     # define ALIGN32_BEG __declspec(align(32))
     # define ALIGN32_END 
 #else
@@ -149,20 +177,20 @@
 #endif
 	#define DIAG_STR(s) #s
 	#define DIAG_JOINSTR(x,y) DIAG_STR(x ## y)
-	#ifdef MSVC_COMPILER
+	#ifdef COMPILER_MSVC
 		#define DIAG_DO_PRAGMA(x) __pragma (#x)
 		#define DIAG_PRAGMA(compiler,x) DIAG_DO_PRAGMA(warning(x))
 	#else
 		#define DIAG_DO_PRAGMA(x)       _Pragma (#x)
 		#define DIAG_PRAGMA(compiler,x) DIAG_DO_PRAGMA(compiler diagnostic x)
 	#endif
-	#if defined(CLANG_COMPILER)
+	#if defined(COMPILER_CLANG)
 		# define DISABLE_WARNING(gcc_unused,clang_option,msvc_unused) DIAG_PRAGMA(clang,push) DIAG_PRAGMA(clang,ignored DIAG_JOINSTR(-W,clang_option))
 		# define ENABLE_WARNING(gcc_unused,clang_option,msvc_unused) DIAG_PRAGMA(clang,pop)
-	#elif defined(MSVC_COMPILER)
+	#elif defined(COMPILER_MSVC)
 		# define DISABLE_WARNING(gcc_unused,clang_unused,msvc_errorcode) DIAG_PRAGMA(msvc,push) DIAG_DO_PRAGMA(warning(disable:##msvc_errorcode))
 		# define ENABLE_WARNING(gcc_unused,clang_unused,msvc_errorcode) DIAG_PRAGMA(msvc,pop)
-	#elif defined(GCC_COMPILER)
+	#elif defined(COMPILER_GCC)
 		#if ((__GNUC__ * 100)+__GNUC_MINOR__) >=406
 			# define DISABLE_WARNING(gcc_option,clang_unused,msvc_unused) DIAG_PRAGMA(GCC,push) DIAG_PRAGMA(GCC,ignored DIAG_JOINSTR(-W,gcc_option))
 			# define ENABLE_WARNING(gcc_option,clang_unused,msvc_unused) DIAG_PRAGMA(GCC,pop)
@@ -171,7 +199,7 @@
 			# define ENABLE_WARNING(gcc_option,clang_option,msvc_unused) DIAG_PRAGMA(GCC,warning DIAG_JOINSTR(-W,gcc_option))
 		#endif
 	#endif
-#if defined(GCC_COMPILER)  
+#if defined(COMPILER_GCC) 
 	#define  DISABLE_MANY_WARNINGS   \
 	DISABLE_WARNING(unknown-pragmas,unknown-pragmas,NOT_USED) \
 	DISABLE_WARNING(pragmas,pragmas,NOT_USED) \
@@ -197,8 +225,10 @@
     DISABLE_WARNING(switch,switch,NOT_USED) \
     DISABLE_WARNING(uninitialized,uninitialized,NOT_USED)\
     DISABLE_WARNING(pedantic,pedantic,NOT_USED) \
-    DISABLE_WARNING(div-by-zero,div-by-zero,NOT_USED)
+    DISABLE_WARNING(div-by-zero,div-by-zero,NOT_USED)\
+    DISABLE_WARNING(constant-logical-operand,constant-logical-operand,NOT_USED) 
 	#define  ENABLE_MANY_WARNINGS   \
+    ENABLE_WARNING(constant-logical-operand,constant-logical-operand,NOT_USED) \
     ENABLE_WARNING(div-by-zero,div-by-zero,NOT_USED) \
     ENABLE_WARNING(pedantic,pedantic,NOT_USED)\
     ENABLE_WARNING(uninitialized,uninitialized,NOT_USED)\
@@ -224,7 +254,7 @@
 	ENABLE_WARNING(unused-variable,unused-variable,NOT_USED)  \
 	ENABLE_WARNING(pragmas,pragmas,NOT_USED) \
 	ENABLE_WARNING(unknown-pragmas,unknown-pragmas,NOT_USED) 
-#elif defined(CLANG_COMPILER)  
+#elif defined(COMPILER_CLANG)||1
 	#define  DISABLE_MANY_WARNINGS  \
 	DISABLE_WARNING(unknown-pragmas,unknown-pragmas,NOT_USED)  \
 	DISABLE_WARNING(pragmas,pragmas,NOT_USED) \
@@ -247,8 +277,12 @@
     DISABLE_WARNING(uninitialized,uninitialized,NOT_USED)\
     DISABLE_WARNING(pedantic,pedantic,NOT_USED) \
     DISABLE_WARNING(typedef-redefinition,typedef-redefinition,NOT_USED) \
-    DISABLE_WARNING(div-by-zero,div-by-zero,NOT_USED)
+    DISABLE_WARNING(div-by-zero,div-by-zero,NOT_USED) \
+	DISABLE_WARNING(unused-but-set-variable,unused-but-set-variable,NOT_USED) \
+    DISABLE_WARNING(constant-logical-operand,constant-logical-operand,NOT_USED) 
 	#define  ENABLE_MANY_WARNINGS  \
+    ENABLE_WARNING(constant-logical-operand,constant-logical-operand,NOT_USED)  \
+    ENABLE_WARNING(unused-but-set-variable,unused-but-set-variable,NOT_USED) \
     ENABLE_WARNING(div-by-zero,div-by-zero,NOT_USED) \
     ENABLE_WARNING(typedef-redefinition,typedef-redefinition,NOT_USED)\
     ENABLE_WARNING(pedantic,pedantic,NOT_USED)\
