@@ -5,6 +5,7 @@
 #ifdef __cplusplus
 extern "C" {
 #endif
+	extern void StdouFlush(void);
 	extern I08 IDE_USER_INTERRUPT;
 	typedef enum   IO_TYPE { MEM_IO,DISK_IO } IO_TYPE;
 	typedef struct FIELD_ITEM {
@@ -12,14 +13,14 @@ extern "C" {
 		DATA_TYPE type;
 		int       ndim;
 		int       dims[5];
-		void** ptr;
+		void**   ptr;
 		int       extra; 
 	} FIELD_ITEM;
 	VOID_PTR GetFieldByIdx(VOID_PTR strucVar,I32 ind);
-	void    GetFieldNameByIdx(VOID_PTR strucVar,I32 ind0,char* str,int buflen);
-	void*   CreateNumVar(DATA_TYPE dtype,int* dims,int ndims,VOIDPTR* data_ptr);
-	void  * CreateNumVector(DATA_TYPE dtype,int length,VOIDPTR* data_ptr);
-	void  * CreateNumMatrix(DATA_TYPE dtype,int Nrow,int Ncol,VOIDPTR* data_ptr);
+	void  GetFieldNameByIdx(VOID_PTR strucVar,I32 ind0,char* str,int buflen);
+	void* CreateNumVar(DATA_TYPE dtype,int* dims,int ndims,VOIDPTR* data_ptr);
+	void* CreateNumVector(DATA_TYPE dtype,int length,VOIDPTR* data_ptr);
+	void* CreateNumMatrix(DATA_TYPE dtype,int Nrow,int Ncol,VOIDPTR* data_ptr);
 	void* CreateF32NumVector(int length,VOIDPTR* data_ptr);
 	void* CreateF32NumMatrix(int Nrow,int Ncol,VOIDPTR* data_ptr);
 	void* CreateF64NumVector(int length,VOIDPTR* data_ptr);
@@ -35,7 +36,7 @@ extern "C" {
 	void AddIntegerAttribute(VOID_PTR listVar,const char* field,I32 value);
 	void RemoveAttribute(VOID_PTR listVar,const char* field);
 	extern  I32   GetConsoleWidth(void);
-	extern  void  printProgress(F32 pct,I32 width,char* buf,I32 firstTimeRun);
+	extern  void  printProgress1(F32 pct,I32 width,char* buf,I32 firstTimeRun);
 	extern  void  printProgress2(F32 pct,F64 time,I32 width,char* buf,I32 firstTimeRun);
 	I32 GetCharArray(void* ptr,char* dst,int n);
 	I32 GetCharVecElem(void* ptr,int idx,char* dst,int n);
@@ -118,15 +119,15 @@ extern "C" {
 #define  IDE_NULL        R_NilValue
 #elif P_INTERFACE==1
 #define  r_printf(...)   PySys_WriteStdout(__VA_ARGS__)
-#define  r_error(...)    printf(__VA_ARGS__)
+#define  r_error(...)    PySys_WriteStderr(__VA_ARGS__)    
 #define  r_warning(...)  printf(__VA_ARGS__)
 #define  r_malloc(x)     PyMem_RawMalloc(x,char)  
 #define  r_free(x)       PyMem_RawFree(x) 
 #define  IDE_NULL        Py_None
 #endif
-	extern char GLOBAL_QUIET_MODE;
-#define q_warning(...)  { if (!GLOBAL_QUIET_MODE) {r_warning(__VA_ARGS__);}}
-#define q_printf(...)   { if (!GLOBAL_QUIET_MODE) {r_printf(__VA_ARGS__);} }
+extern char GLOBAL_PRNT_WARNING;
+#define q_warning(...)  { if (GLOBAL_PRNT_WARNING) {r_warning(__VA_ARGS__);} }
+#define q_printf(...)   { if (GLOBAL_PRNT_WARNING) {r_printf(__VA_ARGS__); } }
 #if R_INTERFACE==1
 	extern  SEXP	getListElement(SEXP list,const char* str);
 	extern  SEXP    getListElement_CaseIn(SEXP list,const char* str);
