@@ -123,7 +123,7 @@ void chol_columwise(F32PTR A,F32PTR U,I64  N,I64 K)
 			Ucol=Ucol - (col - 1);  
 			U=U - (col - 1)+N; 
 		}
-		Ucol[COL - 1]=sqrt( A[COL-1] - SUM_UxU);
+		Ucol[COL - 1]=(F32) sqrt( A[COL-1] - SUM_UxU);
 	}
 }
 void chol_columwise_v2( F32PTR A,F32PTR U,I64  N,I64 K )
@@ -140,11 +140,11 @@ void chol_columwise_v2( F32PTR A,F32PTR U,I64  N,I64 K )
 			for (I32 row=1; row < col;++row)		
 				sum+=U[row - 1] * Ucol[row - 1];
 			F64 res=(A[col -1]-sum)/U[col -1];
-			Ucol[col - 1]=res;
+			Ucol[col - 1]=(F32) res;
 			SUM_Ucol_x_Ucol+=res * res;	
 			U+=N;								 
 		}
-		Ucol[COL -1]=sqrt(A[COL -1]-SUM_Ucol_x_Ucol);
+		Ucol[COL -1]=(F32) sqrt(A[COL -1]-SUM_Ucol_x_Ucol);
 	}
 }
 void chol_rowwise( F32PTR A,F32PTR U,I64  N,I64 K ) { 
@@ -154,7 +154,7 @@ void chol_rowwise( F32PTR A,F32PTR U,I64  N,I64 K ) {
 		U=U_base+(ROW-1)*N;         
 		A=A_base+(ROW-1)*N ;        
 		F64 sum=0.0; 	for (I32 row=1; row < ROW;++row) {sum+=U[row-1]* U[row-1]; }
-		F32 Ukk=sqrt(A[ROW-1]-sum);				
+		F32 Ukk=(F32) sqrt(A[ROW-1]-sum);
 		F32 Ukk_inv=1.f/Ukk;
 		U[ROW - 1]=Ukk;
 		F32PTR Ucurcol=U;			
@@ -179,12 +179,12 @@ void chol_addCol(F32PTR A,F32PTR U,I64 N,I64 K0,I64 K1)
 			F64 sum=0.f;
 			for (I32 row=1; row < col; row++)	{sum+=(*U++)* (*Ucol++);}			
 			F64 Uk=(A[col - 1] - sum)/(*U);
-			*Ucol=Uk;
+			*Ucol=(F32) Uk;
 			SUM+=Uk * Uk;
 			Ucol=Ucol - (col - 1);
 			U=U - (col - 1)+N;
 		}
-		Ucol[COL - 1]=sqrt(A[COL - 1] - SUM);
+		Ucol[COL - 1]=(F32) sqrt(A[COL - 1] - SUM);
 	}
 }
 void inplace_chol(F32PTR A,I64  N,I64 K)
@@ -196,7 +196,7 @@ void inplace_chol(F32PTR A,I64  N,I64 K)
 		{	F64 sum=0.f;
 			for (I64 row=1; row < COL; row++) { sum+=A[row - 1] * A[row - 1]; };			
 			F64  Ukk=sqrt(A[COL-1] - sum);			
-			A[COL - 1]=Ukk;
+			A[COL - 1]=(F32) Ukk;
 			Ukk_inv=1.f/Ukk;
 		}		
 		F32PTR U_curCol_base=A;
@@ -772,85 +772,6 @@ void simple_linear_regression_nan(F32PTR Y,F32PTR X,int N,F32PTR Yfit,F32PTR Yer
 		}
 	}
 }
-void update_XtX_from_Xnewterm(F32PTR X,F32PTR Xnewterm,F32PTR XtX,F32PTR XtXnew,NEWCOLINFO * new ) {
-	I32 k1=new->k1;
-	I32 k2_old=new->k2_old;
-	I32 k2_new=new->k2_new;
-	I32 Knewterm=new->Knewterm; 
-	I32 KOLD=new->KOLD;
-	I32 KNEW=new->KNEW;
-	I32 N=new->N;
-	I32 Nlda=new->Nlda;
-	for (I32 i=1; i < k1; i++) SCPY(i,XtX+(i - 1L) * KOLD,XtXnew+(i - 1L) * KNEW);
-	if (Knewterm !=0) {
-		FILL0(XtXnew+(k1 - 1) * KNEW,(KNEW - k1+1) * KNEW); 
-		if (k1 > 1) {
-			r_cblas_sgemm(CblasColMajor,CblasTrans,CblasNoTrans,k1 - 1,Knewterm,N,1.0f,
-				X,Nlda,
-				Xnewterm,Nlda,0.f,
-				XtXnew+(k1 - 1L) * KNEW,KNEW);
-		}
-		r_cblas_sgemm(CblasColMajor,CblasTrans,CblasNoTrans,
-			Knewterm,Knewterm,N,1.0,
-			Xnewterm,Nlda,
-			Xnewterm,Nlda,0.f,
-			XtXnew+(k1 - 1) * KNEW+k1 - 1,KNEW);
-	}
-	if (k2_old !=KOLD) {
-		for (I32 kold=k2_old+1,knew=k2_new+1; kold <=KOLD; kold++,knew++) {
-			F32PTR ColStart_old=XtX+(kold - 1) * KOLD;
-			F32PTR ColStart_new=XtXnew+(knew - 1) * KNEW;
-			SCPY(k1 - 1,ColStart_old,ColStart_new); 
-			SCPY(kold - k2_old,ColStart_old+(k2_old+1) - 1,ColStart_new+(k2_new+1) - 1); 
-		}
-		if (Knewterm !=0) {
-			r_cblas_sgemm(CblasColMajor,CblasTrans,CblasNoTrans,
-				Knewterm,(KOLD - k2_old),N,1.0,
-				Xnewterm,Nlda,
-				X+(k2_old+1 - 1) * Nlda,Nlda,0.0,
-				XtXnew+(k2_new+1 - 1) * KNEW+k1 - 1,KNEW);
-		}
-	}
-}
-void update_XtY_from_Xnewterm(F32PTR Y,F32PTR Xnewterm,F32PTR XtY,F32PTR XtYnew,NEWCOLINFO* new,I32 q) {
-	I32 k1=new->k1;
-	I32 k2_old=new->k2_old;
-	I32 k2_new=new->k2_new;
-	I32 Knewterm=new->Knewterm;
-	I32 N=new->N;
-	I32 Nlda=new->Nlda;
-	I32 KOLD=new->KOLD;
-	I32 KNEW=new->KNEW;
-	if (q==1) {
-		if (k1 > 1)       SCPY(k1 - 1,XtY,XtYnew);
-		if (Knewterm > 0) { 
-				r_cblas_sgemv(CblasColMajor,CblasTrans,N,Knewterm,1.f,
-						Xnewterm,Nlda,
-						Y,1L,0.f,
-					    XtYnew+k1 - 1,1L);
-		}
-		if (k2_old !=KOLD) SCPY(KNEW - k2_new,XtY+(k2_old+1L) - 1L,XtYnew+(k2_new+1) - 1);
-	}
-	else {
-		if (k1 > 1) {
-			for (I32 c=0; c < q;++c) {
-				SCPY(k1 - 1,XtY+KOLD * c,XtYnew+KNEW * c);
-			}
-		}
-		if (Knewterm > 0) {
-			r_cblas_sgemm(CblasColMajor,CblasTrans,CblasNoTrans, 
-				Knewterm,q,N,1.f, 
-				Xnewterm,Nlda,
-				Y,N,0.f,
-				XtYnew+k1 -1,KNEW);
-		}
-		if (k2_old !=KOLD) {
-			for (I32 c=0; c < q;++c) {
-				SCPY(KNEW - k2_new,XtY+(k2_old+1L) - 1L+KOLD * c,XtYnew+(k2_new+1) - 1+KNEW * c);
-			}
-		}
-	}
-}
 void get_parts_for_newinfo(NEWCOLINFOv2* new) {
 	int Knewterm=0;
 	int Kbase_dst=1;
@@ -1046,21 +967,26 @@ void swap_elem_bands(NEWCOLINFOv2* new,void *x,void *xnew,I32 elemSize) {
 	}
 }
 void shift_lastcols_within_matrix(F32PTR X,I32 N,I32 Kstart,I32 Kend,I32 Knewstart) {
-	if (Knewstart==Kstart) {
+	int offset=Knewstart - Kstart;
+	if (offset==0) {
 		return;
 	}
-	int j=Knewstart - Kstart;
-	if (j < 0||Knewstart > Kend) {
-		r_cblas_scopy((Kend - Kstart+1) * N,X+(Kstart - 1) * N,1,X+(Knewstart - 1) * N,1);
-	} else {
+	int Kseg=Kend - Kstart+1;
+	if ( offset>=Kseg||offset <=-Kseg) { 
+		r_cblas_scopy(Kseg * N,X+(Kstart - 1) * N,1,X+(Knewstart - 1) * N,1);
+	}	
+	else if (offset < 0) { 
+		memmove( X+(Knewstart - 1) * N,X+(Kstart - 1) * N,Kseg * N * sizeof(F32));
+	}
+    else {  
 		I32 segStartIdx=Kend+1;
 		while (_True_) {
-			segStartIdx=segStartIdx - j;
+			segStartIdx=segStartIdx -offset;
 			if (segStartIdx > Kstart) {
-				SCPY(j * N,X+(segStartIdx - 1) * N,X+((segStartIdx+j) - 1) * N);				
+				SCPY(offset * N,X+(segStartIdx - 1) * N,X+((segStartIdx+offset) - 1) * N);
 			} else {
-				j=(segStartIdx+j) - Kstart;
-				SCPY(j * N,X+(Kstart - 1) * N,X+(Knewstart - 1) * N);
+				int Kremaining=(segStartIdx+offset) - Kstart;
+				SCPY(Kremaining * N,X+(Kstart - 1) * N,X+(Knewstart - 1) * N);
 				break;
 			}
 		}
