@@ -1,9 +1,8 @@
 #include "abc_000_macro.h"
 #include "abc_000_warning.h"
-#if defined(MSVC_COMPILER)
+#if defined(COMPILER_MSVC)
 #include "intrin.h"                
 #endif
-#include <stdio.h>	               
 #include <string.h>	               
 #include <time.h>
 #include <math.h>
@@ -18,6 +17,7 @@
 #include "abc_rand.h"
 #include "abc_vec.h"   
 #include "abc_math.h"  
+#include <stdio.h>	               
 #include "globalvars.h"  
 #include "beastv2_header.h"
 #include "beastv2_func.h" 
@@ -26,21 +26,18 @@
 #include "beastv2_xxyy_allocmem.h" 
 #include "beastv2_io.h" 
 #define LOCAL(...) do{ __VA_ARGS__ } while(0);
-int beast2_main_corev4_mthrd(void* dummy)
-{
+#define  DEBUG_MODE  0
+int beast2_main_corev4_mthrd(void* dummy) {
 	MemPointers MEM=(MemPointers){.init=mem_init,};
 	MEM.init(&MEM);
+#if DEBUG_MODE==1
+	MEM.checkHeader=1;
+#endif
 	VSLStreamStatePtr stream;
 	const BEAST2_OPTIONS_PTR	opt=GLOBAL_OPTIONS;
 	const BEAST2_EXTRA          extra=opt->extra;
 	typedef int QINT;
 	const QINT  q=opt->io.q;
-	CI_PARAM     ciParam={0,};
-	CI_RESULT    ci[MAX_NUM_BASIS];
-	if (extra.computeCredible) {
-		ConstructCIStruct(	opt->mcmc.credIntervalAlphaLevel,opt->mcmc.samples,opt->io.N*opt->io.q,  
-							opt->prior.numBasis,&MEM,&extra.fastCIComputation,&ciParam,ci );  
-	}
 	BEAST2_MODEL  MODEL={0,};
 	AllocInitModelMEM(&MODEL,opt,&MEM);
 	LOCAL( 	
@@ -54,46 +51,68 @@ int beast2_main_corev4_mthrd(void* dummy)
 	const U32PTR  RND32_END=RND32+MAX_RAND_NUM - 7;
 	const U16PTR  RND16_END=RND16+MAX_RAND_NUM * 2 - 7;
 	const U08PTR  RND08_END=RND08+MAX_RAND_NUM * 4 - 7 -3;     
-	const F32PTR  RNDGAMMA_END=RNDGAMMA+MAX_RAND_NUM - MODEL.nPrec-1L;
+	const F32PTR  RNDGAMMA_END=RNDGAMMA+MAX_RAND_NUM - MODEL.precState.nPrecGrp-1L;
 	const F32PTR Xt_mars;
 	const F32PTR Xnewterm;      
 	const F32PTR Xt_zeroBackup; 
 	AllocateXXXMEM(&Xt_mars,&Xnewterm,&Xt_zeroBackup,&MODEL,opt,&MEM);
 	BEAST2_YINFO     yInfo;
 	AllocateYinfoMEM(&yInfo,opt,&MEM);
-	BEAST2_RESULT resultChain={ NULL,},result={ NULL,};
+	const BEAST2_RESULT resultChain={ NULL,},result={ NULL,};
 	BEAST2_Result_AllocMEM(&resultChain,opt,&MEM); 	
 	BEAST2_Result_AllocMEM(&result,opt,&MEM);
+	const   I32  NumCIVars=MODEL.NUMBASIS+opt->extra.computeTrendSlope;
+	 CI_PARAM     ciParam={ 0,};
+	CI_RESULT    ci[MAX_NUM_BASIS+1];
+	if (extra.computeCredible) {
+		ConstructCIStruct(opt->mcmc.credIntervalAlphaLevel,opt->mcmc.samples,opt->io.N * opt->io.q,  
+			              NumCIVars,&MEM,&extra.fastCIComputation,&ciParam,ci);
+	}
 	if (extra.computeCredible) {
 		I32  Npad=(opt->io.N+7)/8 * 8;
 		I32  XnewtermOffset=0;
 		Npad=opt->io.N;    
-		for (I32 i=0; i < MODEL.NUMBASIS; i++) {
-			if (MODEL.b[i].type==SEASONID||MODEL.b[i].type==DUMMYID||MODEL.b[i].type==SVDID)
-				ci[i].result=resultChain.sCI,		         
-			    ci[i].newDataRow=Xnewterm+XnewtermOffset;	 
-			else if (MODEL.b[i].type==TRENDID)
-				ci[i].result=resultChain.tCI,               
-			    ci[i].newDataRow=Xnewterm+XnewtermOffset;    
-			else if (MODEL.b[i].type==OUTLIERID)
-				ci[i].result=resultChain.oCI,               
-			    ci[i].newDataRow=Xnewterm+XnewtermOffset;     
+		I08 hasSeasonCmpnt=opt->prior.basisType[0]==SEASONID||opt->prior.basisType[0]==DUMMYID||opt->prior.basisType[0]==SVDID;
+		I08 hasTrendCmpnt=1;
+		I08 hasOutlierCmpnt=opt->prior.basisType[opt->prior.numBasis - 1]==OUTLIERID;
+		int numCIVars=0;
+		if (hasSeasonCmpnt) {
+			ci[numCIVars].result=resultChain.sCI;		         
+			ci[numCIVars].newDataRow=Xnewterm+XnewtermOffset;	 
+			numCIVars++;
+			XnewtermOffset+=Npad * q;   
+		}
+		if (hasTrendCmpnt) {
+			ci[numCIVars].result=resultChain.tCI;               
+			ci[numCIVars].newDataRow=Xnewterm+XnewtermOffset;    
+			numCIVars++;
+			XnewtermOffset+=Npad * q;   
+		}	 
+		if (hasOutlierCmpnt) {
+		  ci[numCIVars].result=resultChain.oCI,               
+		  ci[numCIVars].newDataRow=Xnewterm+XnewtermOffset;     
+		  numCIVars++;
+		  XnewtermOffset+=Npad * q;   
+		}
+		if (opt->extra.computeTrendSlope) {
+			ci[numCIVars].result=resultChain.tslpCI;           
+			ci[numCIVars].newDataRow=Xnewterm+XnewtermOffset;    
+			numCIVars++;
 			XnewtermOffset+=Npad * q;   
 		}
 	} 
 	const CORESULT coreResults[MAX_NUM_BASIS];
 	SetupPointersForCoreResults(coreResults,MODEL.b,MODEL.NUMBASIS,&resultChain);
-	const BEAST2_HyperPar  hyperPar={.alpha_1=opt->prior.alpha1,.alpha_2=opt->prior.alpha2,.del_1=opt->prior.delta1,.del_2=opt->prior.delta2};
+	const BEAST2_HyperPar  hyperPar={ .alpha_1=opt->prior.alpha1,.alpha_2=opt->prior.alpha2,.del_1=opt->prior.delta1,.del_2=opt->prior.delta2};
 	InitTimerFunc();
 	StartTimer();
 	SetBreakPointForStartedTimer();
 	const PREC_FUNCS precFunc;
 	SetUpPrecFunctions(opt->prior.precPriorType,opt->io.q,&precFunc);
-	if (extra.printProgressBar) {
+	if (extra.printProgress) {
 		F32 frac=0.0; I32 firstTimeRun=1;
 	}
-	#undef  __DEBUG__ 
-	#ifdef __DEBUG__
+	#if DEBUG_MODE==1
 		I32    N=opt->io.N;
 		I32    Npad=(N+7)/8 * 8; Npad=N;
 		F32PTR flagSat=MyALLOC0(MEM,N,I32,64);
@@ -105,10 +124,12 @@ int beast2_main_corev4_mthrd(void* dummy)
 	const U32  MCMC_BURNIN=opt->mcmc.burnin;
 	const U32  MCMC_CHAINNUM=opt->mcmc.chainNumber;
 	const U16  SEASON_BTYPE=opt->prior.seasonBasisFuncType;
-	const U16  GROUP_MatxMat=(MODEL.sid <0||opt->prior.seasonBasisFuncType !=3 )
-						       && (MODEL.vid < 0||opt->prior.trendBasisFuncType !=3)
-							   && (MODEL.tid<0||opt->prior.trendBasisFuncType!=2)
-		                       && ( MODEL.oid<0||opt->prior.outlierBasisFuncType!=2);
+	void (*Update_XtX_from_Xnewterm)(F32PTR X,F32PTR Xnewterm,F32PTR XtX,F32PTR XtXnew,NEWTERM * NEW,BEAST2_MODEL * MODEL);
+	void (*Update_XtY_from_Xnewterm)(F32PTR Y,F32PTR Xnewterm,F32PTR XtY,F32PTR XtYnew,NEWTERM * new,I32 q);
+	U16  GROUP_MatxMat=(MODEL.sid < 0||opt->prior.seasonBasisFuncType  !=3 ) && (MODEL.vid < 0||opt->prior.trendBasisFuncType   !=3 )
+		              && (MODEL.tid < 0||opt->prior.trendBasisFuncType   !=2 ) && (MODEL.oid < 0||opt->prior.outlierBasisFuncType !=2 );
+	Update_XtX_from_Xnewterm=GROUP_MatxMat ? Update_XtX_from_Xnewterm_ByGroup : Update_XtX_from_Xnewterm_NoGroup;
+	Update_XtY_from_Xnewterm=GROUP_MatxMat ? Update_XtY_from_Xnewterm_ByGroup : Update_XtY_from_Xnewterm_NoGroup;
 	for (U32 pixelIndex=1; pixelIndex <=NUM_PIXELS; pixelIndex++)
 	{
 		pthread_mutex_lock(&mutex);
@@ -122,11 +143,11 @@ int beast2_main_corev4_mthrd(void* dummy)
 		BEAST2_fetch_timeSeries(&yInfo,pixelIndex,MEMBUF,&(opt->io));
 		F32PTR  Xtmp=Xt_mars;
 		U08     skipCurrentPixel=BEAST2_preprocess_timeSeries(&yInfo,MODEL.b,Xtmp,opt);		
-		#ifdef __DEBUG__
-			I32 accS[5]={ 0,0,0,0,0 },accT[5]={ 0,0,0,0,0 };
-			I32 flagS[5]={ 0,0,0,0,0 },flagT[5]={ 0,0,0,0,0 };
+        #if DEBUG_MODE==1
+			I32 accS[5]={ 0,},accT[5]={ 0,};
+			I32 flagS[5]={ 0,},flagT[5]={ 0,};
 			for (int i=0; i < yInfo.nMissing; i++) { flagSat[yInfo.rowsMissing[i]]=getNaN();}
-		#endif
+        #endif
 		#define __START_IF_NOT_SKIP_TIMESESIRIES__    
 		#define __END_IF_NOT_SKIP_TIMESESIRIES__                        
 		__START_IF_NOT_SKIP_TIMESESIRIES__  
@@ -151,288 +172,223 @@ int beast2_main_corev4_mthrd(void* dummy)
 		for ( U32 chainNumber=0;  chainNumber < MCMC_CHAINNUM; chainNumber++)
 		{
 			const I32  N=opt->io.N; 
-			const I32  Npad=N;
+			const I32  Npad=N;  (N+7)/8 * 8; 
 			const I32  Npad16=(N+15)/16 * 16;	
 			{   
-				GenarateRandomBasis(MODEL.b,MODEL.NUMBASIS,N,&RND);
-				MODEL.b[0].Kbase=0;                           
-				UpdateBasisKbase(MODEL.b,MODEL.NUMBASIS,0);	
-				precFunc.GetNumTermsPerPrecGrp(&MODEL); 
-				precFunc.GetXtXPrecDiag(&MODEL);        
-				CvtKnotsToBinVec(MODEL.b,MODEL.NUMBASIS,N,&yInfo);
-				if (q==1) {
-					BEAST2_EvaluateModel(&MODEL.curr,MODEL.b,Xt_mars,N,MODEL.NUMBASIS,&yInfo,&hyperPar,&opt->prior.precValue,&stream); 
-				} else 	{
-					MR_EvaluateModel(    &MODEL.curr,MODEL.b,Xt_mars,N,MODEL.NUMBASIS,&yInfo,&hyperPar,&opt->prior.precValue,&stream);
-				}
+				GenarateRandomBasis(MODEL.b,MODEL.NUMBASIS,N,&RND,&yInfo);
+				BEAST2_EvaluateModel(&MODEL.curr,MODEL.b,Xt_mars,N,MODEL.NUMBASIS,&yInfo,&hyperPar,&MODEL.precState,&precFunc); 
 			}
 			{
-				BEAST2_Result_FillMEM(&resultChain,opt,0);
+				CvtKnotsToBinVec(MODEL.b,MODEL.NUMBASIS,N,&yInfo);
 				memset(MODEL.extremePosVec,1,N);
 				for (I32 i=0; i < yInfo.nMissing;++i) MODEL.extremePosVec[yInfo.rowsMissing[i]]=0;
 				MODEL.extremPosNum=yInfo.n;
+				f32_fill_val(1e30,MODEL.deviation,N);
+				for (I32 i=0; i < yInfo.nMissing;++i) MODEL.deviation[yInfo.rowsMissing[i]]=getNaN();
+				MODEL.avgDeviation[0]=1.0;
+				BEAST2_Result_FillMEM(&resultChain,opt,0);
+				MODEL.b[0].Kbase=0;                           
+				UpdateBasisKbase(MODEL.b,MODEL.NUMBASIS,0);	
 			}
 			U32 ite=0;
 			U32 sample=0;
 			if (extra.computeCredible) { 
-				for (int i=0; i < MODEL.NUMBASIS; i++) {
+				for (int i=0; i < NumCIVars; i++) {
 					ci[i].samplesInserted=0;
 				} 
 			} 
-			PROP_DATA PROPINFO={.N=N,.Npad16=Npad16,.samples=&sample,
-				                  .keyresult=coreResults,.mem=Xnewterm,.model=&MODEL, 
-				                  .pRND=&RND,.yInfo=&yInfo,.nSample_ExtremVecNeedUpdate=1L,       
-								  .sigFactor=opt->prior.sigFactor,.outlierSigFactor=opt->prior.outlierSigFactor,
-			}; 
-			NEWTERM        NEW;     
-			NEWCOLINFO     NewCol; 
-			NewCol.N=N;
-			NewCol.Nlda=Npad;
+			PROP_DATA PROPINFO={ .N=N,.Npad16=Npad16,.samples=&sample,.keyresult=coreResults,.mem=Xnewterm,.model=&MODEL,
+							  .pRND=&RND,.yInfo=&yInfo,.sigFactor=opt->prior.sigFactor,.outlierSigFactor=opt->prior.outlierSigFactor,
+							  .nSample_DeviationNeedUpdate=1L,.shallUpdateExtremVec=0L, 
+				               .numBasisWithoutOutlier=MODEL.NUMBASIS - (opt->prior.basisType[MODEL.NUMBASIS - 1]==OUTLIERID),};
+			NEWTERM   NEW={ .newcols={.N=N,.Nlda=Npad} };
 			I32 numBadIterations=0;
 			while (sample < MCMC_SAMPLES)
 			{
 				ite++;
-				if (RND.rnd32    >=RND32_END)    {r_viRngUniformBits32(VSL_RNG_METHOD_UNIFORMBITS32_STD,stream,(RND.rnd32 - RND32),(U32PTR)RND32);	                               RND.rnd32=RND32;   }
-				if (RND.rnd16    >=RND16_END)    {r_viRngUniformBits32(VSL_RNG_METHOD_UNIFORMBITS32_STD,stream,((char*)RND.rnd16 - (char*)RND16+3)/sizeof(U32),(U32PTR)RND16); RND.rnd16=RND16;   }
-				if (RND.rnd08    >=RND08_END)    {r_viRngUniformBits32(VSL_RNG_METHOD_UNIFORMBITS32_STD,stream,((char*)RND.rnd08 - (char*)RND08+3)/sizeof(U32),(U32PTR)RND08); RND.rnd08=RND08;   }
-				if (RND.rndgamma >=RNDGAMMA_END) {r_vsRngGamma(VSL_RNG_METHOD_GAMMA_GNORM_ACCURATE,stream,MAX_RAND_NUM,RNDGAMMA,(hyperPar.alpha_1+yInfo.n*0.5f),0.f,1.f);    RND.rndgamma=RNDGAMMA;}
+				if (RND.rnd32    >=RND32_END)    {r_viRngUniformBits32(VSL_RNG_METHOD_UNIFORMBITS32_STD,stream,(RND.rnd32 - RND32),(U32PTR)RND32 ); RND.rnd32=RND32;   }
+				if (RND.rnd16    >=RND16_END)    {r_viRngUniformBits32(VSL_RNG_METHOD_UNIFORMBITS32_STD,stream,((char*)RND.rnd16 - (char*)RND16+3)/sizeof(U32),(U32PTR)RND16 ); RND.rnd16=RND16;   }
+				if (RND.rnd08    >=RND08_END)    {r_viRngUniformBits32(VSL_RNG_METHOD_UNIFORMBITS32_STD,stream,((char*)RND.rnd08 - (char*)RND08+3)/sizeof(U32),(U32PTR)RND08 ); RND.rnd08=RND08;   }
+				if (RND.rndgamma >=RNDGAMMA_END) {r_vsRngGamma(VSL_RNG_METHOD_GAMMA_GNORM_ACCURATE,stream,MAX_RAND_NUM,RNDGAMMA,(hyperPar.alpha_1+yInfo.n*0.5f),0.f,1.f  ); RND.rndgamma=RNDGAMMA;}
 				BEAST2_BASIS_PTR basis=MODEL.b+MODEL.PickBasisID(&PROPINFO); 
-				basis->Propose(basis,&NEW,&NewCol,&PROPINFO); 
-				#ifdef __DEBUG__
+				basis->Propose(basis,&NEW,&PROPINFO);
+				#if  DEBUG_MODE==1
 					I32 basisIdx=basis - MODEL.b;		
 					flagSat[NEW.newKnot - 1]+=basisIdx==0 && (NEW.jumpType==BIRTH||NEW.jumpType==MOVE);					 
 					if (basisIdx==0)++(flagS[NEW.jumpType]);
 					else++(flagT[NEW.jumpType]); 
+					MEM.verify_header(&MEM);
 				#endif
 				I32 Knewterm=0;				
 				for (I32 i=0; i < NEW.numSeg; i++) { 
 					I32 kterms=NEW.SEG[i].K=basis->GenTerms(Xnewterm+Npad*Knewterm,N,&NEW.SEG[i],&(basis->bConst));					
 					Knewterm+=kterms;
 				} 
-				NewCol.k2_new=NewCol.k1+Knewterm - 1L;	
-				NewCol.k1+=basis->Kbase;		
-				NewCol.k2_old+=basis->Kbase;
-				NewCol.k2_new+=basis->Kbase;
-				I32 KOLD=MODEL.curr.K;                         
-				I32 KNEW=KOLD+NewCol.k2_new - NewCol.k2_old; 
-				NewCol.Knewterm=Knewterm;
-				NewCol.KOLD=KOLD;
-				NewCol.KNEW=KNEW;				
+				NEW.newcols.k2_new=NEW.newcols.k1+Knewterm - 1L;	
+				NEW.newcols.k1+=basis->Kbase;		
+				NEW.newcols.k2_old+=basis->Kbase;
+				NEW.newcols.k2_new+=basis->Kbase;
+				int KOLD=MODEL.curr.K;
+				int KNEW=MODEL.curr.K+NEW.newcols.k2_new - NEW.newcols.k2_old;
+				NEW.newcols.Knewterm=Knewterm;
+				NEW.newcols.KOLD=KOLD; 
+				NEW.newcols.KNEW=KNEW; 
 				if (yInfo.nMissing > 0 && Knewterm > 0 )  
 				f32_mat_multirows_extract_set_by_scalar(Xnewterm,Npad,Knewterm,Xt_zeroBackup,yInfo.rowsMissing,yInfo.nMissing,0.0f);
-				if (!GROUP_MatxMat) {
-					update_XtX_from_Xnewterm(Xt_mars,Xnewterm,MODEL.curr.XtX,MODEL.prop.XtX,&NewCol);
-					update_XtY_from_Xnewterm(yInfo.Y,Xnewterm,MODEL.curr.XtY,MODEL.prop.XtY,&NewCol,q);
+			    #if DEBUG_MODE==1
+					MEM.verify_header(&MEM);
+				#endif
+	 			Update_XtX_from_Xnewterm(Xt_mars,Xnewterm,MODEL.curr.XtX,MODEL.prop.XtX,&NEW,&MODEL); 
+				Update_XtY_from_Xnewterm(yInfo.Y,Xnewterm,MODEL.curr.XtY,MODEL.prop.XtY,&NEW,q);
+				if (1L) {
+					for (I32 i=1; i < NEW.newcols.k1; i++) 	SCPY(i,MODEL.curr.cholXtX+(i - 1) * KOLD,MODEL.prop.cholXtX+(i - 1) * KNEW);
+					precFunc.SetPropPrecXtXDiag_NtermsPerGrp(&MODEL,basis,&NEW); 
+					precFunc.chol_addCol(MODEL.prop.XtX+(NEW.newcols.k1 - 1) * KNEW,MODEL.prop.cholXtX,MODEL.prop.precXtXDiag,KNEW,NEW.newcols.k1,KNEW);
 				}
-				else {
-					for (I32 i=1; i < NewCol.k1; i++) SCPY(i,MODEL.curr.XtX+(i - 1L) * KOLD,MODEL.prop.XtX+(i - 1L) * KNEW);
-					if (Knewterm !=0) {
-						FILL0(MODEL.prop.XtX+(NewCol.k1 - 1) * KNEW,(KNEW - NewCol.k1+1) * KNEW); 
-						if (NewCol.k1 > 1) {
-							BEAST2_BASESEG* _segInfo=(BEAST2_BASESEG*)(Xnewterm+Knewterm * Npad);
-							I32             _numBands=GetInfoBandList(_segInfo,&MODEL,NewCol.k1 - 1);
-							MatxMat(_segInfo,_numBands,Xt_mars,
-								NEW.SEG,NEW.numSeg,Xnewterm,
-								MODEL.prop.XtX+(NewCol.k1 - 1L) * KNEW,N,KNEW);
-						}
-						XtX_ByGroup(NEW.SEG,NEW.numSeg,Xnewterm,MODEL.prop.XtX+(NewCol.k1 - 1) * KNEW+NewCol.k1 - 1,N,KNEW);
+				MODEL.prop.K=KNEW;
+				precFunc.ComputeMargLik(&MODEL.prop,&MODEL.precState,&yInfo,&hyperPar);
+				if (IsNaN(MODEL.prop.marg_lik)||IsInf(MODEL.prop.marg_lik)) {
+					if (++numBadIterations < 15) {
+						precFunc.IncreasePrecValues(&MODEL);
+						precFunc.SetPrecXtXDiag(MODEL.curr.precXtXDiag,MODEL.b,MODEL.NUMBASIS,&MODEL.precState);
+						precFunc.chol_addCol(MODEL.curr.XtX,MODEL.curr.cholXtX,MODEL.curr.precXtXDiag,MODEL.curr.K,1L,MODEL.curr.K);
+						precFunc.ComputeMargLik(&MODEL.curr,&MODEL.precState,&yInfo,&hyperPar);
+						continue;
+					} else {
+						skipCurrentPixel=2;
+						break;
 					}
-					if (NewCol.k2_old !=KOLD) {
-						for (I32 kold=NewCol.k2_old+1,knew=NewCol.k2_new+1; kold <=KOLD; kold++,knew++) {
-							F32PTR ColStart_old=MODEL.curr.XtX+(kold - 1) * KOLD;
-							F32PTR ColStart_new=MODEL.prop.XtX+(knew - 1) * KNEW;
-							SCPY(NewCol.k1 - 1,ColStart_old,ColStart_new); 
-							SCPY(kold - NewCol.k2_old,ColStart_old+(NewCol.k2_old+1) - 1,ColStart_new+(NewCol.k2_new+1) - 1); 
-						}
-						if (Knewterm !=0) {
-							BEAST2_BASESEG* _segInfo=(BEAST2_BASESEG*)(Xnewterm+Knewterm * Npad);
-							I32             _numBands=GetInfoBandList_post(_segInfo,&MODEL,NewCol.k2_old+1);
-							MatxMat(NEW.SEG,NEW.numSeg,Xnewterm,_segInfo,_numBands,Xt_mars+NewCol.k2_old * Npad,
-									MODEL.prop.XtX+(NewCol.k2_new+1 - 1) * KNEW+NewCol.k1 - 1,N,KNEW);
-						}
-					}
-					if (q==1) {
-						if (NewCol.k1 > 1)         SCPY(NewCol.k1 - 1,MODEL.curr.XtY,MODEL.prop.XtY);
-						if (Knewterm > 0)  	          MatxVec(NEW.SEG,NEW.numSeg,Xnewterm,yInfo.Y,MODEL.prop.XtY+NewCol.k1 - 1,N);
-						if (NewCol.k2_old !=KOLD) SCPY(KNEW - NewCol.k2_new,MODEL.curr.XtY+(NewCol.k2_old+1L) - 1L,MODEL.prop.XtY+(NewCol.k2_new+1) - 1);
-					}	else {
-						if (NewCol.k1 > 1) {
-							for (I32 c=0; c < q;++c) {
-								SCPY(NewCol.k1 - 1,MODEL.curr.XtY+KOLD * c,MODEL.prop.XtY+KNEW * c);
-							}
-						}
-						if (Knewterm > 0) {
-							r_cblas_sgemm(CblasColMajor,CblasTrans,CblasNoTrans,Knewterm,q,N,1.f,Xnewterm,Npad,yInfo.Y,N,0.f,
-								MODEL.prop.XtY+NewCol.k1 - 1,KNEW);
-						}
-						if (NewCol.k2_old !=KOLD) {
-							for (I32 c=0; c < q;++c) {
-								SCPY(KNEW - NewCol.k2_new,MODEL.curr.XtY+(NewCol.k2_old+1L) - 1L+KOLD * c,MODEL.prop.XtY+(NewCol.k2_new+1) - 1+KNEW * c);
-							}
-						}
-					} 
+				}	else {
+					numBadIterations=0;
+				} 
+				F32 delta_lik=MODEL.prop.marg_lik - MODEL.curr.marg_lik;
+				if   ( !(NEW.jumpType==MOVE||basis->type==OUTLIERID) ) {
+					F32 factor=basis->ModelPrior(basis,&NEW.newcols,&NEW); 
+					delta_lik+=factor;
 				}
-				if (1L) {		
-					for (I32 i=1; i<NewCol.k1; i++) 
-						SCPY(i,MODEL.curr.cholXtX+(i-1)*KOLD,MODEL.prop.cholXtX+(i-1)*KNEW);
-					precFunc.UpdateXtXPrec_nTermsPerGrp(&MODEL,basis,&NEW,&NewCol); 
-					precFunc.chol_addCol(  MODEL.prop.XtX+(NewCol.k1-1)*KNEW,
-							               MODEL.prop.cholXtX,
-							               MODEL.prop.precXtXDiag,KNEW,NewCol.k1,KNEW);
-				   MODEL.prop.K=KNEW;
-				   precFunc.ComputeMargLik(&MODEL.prop,&MODEL,&yInfo,&hyperPar);
-				   if ( IsNaN(MODEL.prop.marg_lik)||IsInf(MODEL.prop.marg_lik ) ) {
-					   if (++numBadIterations < 15) {
-						   precFunc.IncreasePrecValues(&MODEL);
-						   precFunc.GetXtXPrecDiag(&MODEL);
-						   precFunc.chol_addCol(MODEL.curr.XtX,MODEL.curr.cholXtX,MODEL.curr.precXtXDiag,MODEL.curr.K,1L,MODEL.curr.K);
-						   precFunc.ComputeMargLik(&MODEL.curr,&MODEL,&yInfo,&hyperPar);
-						   continue;
-					   }  else {
-						   skipCurrentPixel=2;
-						   break;
-					   }					   
-				   }  else {
-					   numBadIterations=0;
-				   } 
-				   if (q==1) {
-					   MODEL.prop.alpha2Q_star[0]=max(MODEL.prop.alpha2Q_star[0],MIN_ALPHA2_VALUE);
-				   }
-				}
-				F32  factor;				
-				if   ( NEW.jumpType==MOVE||basis->type==OUTLIERID) 	factor=0.;
-				else { factor=basis->ModelPrior(basis,&NewCol,&NEW); }
-				F32 delta_lik=MODEL.prop.marg_lik - MODEL.curr.marg_lik+factor;
 				I08     acceptTheProposal;
-				if      (delta_lik >   0)   acceptTheProposal=1;
-				else if (delta_lik < -23) 	acceptTheProposal=0;				
+				if      (delta_lik >   0.0f)   acceptTheProposal=1;
+				else if (delta_lik < -23.0f)   acceptTheProposal=0;				
 				else {				 
 					F32    expValue=fastexp(delta_lik);
-					if     (delta_lik > -0.5) 	acceptTheProposal=*(RND.rnd08)++< expValue * 255.0f;
-					else if(delta_lik > -5  )   acceptTheProposal=*(RND.rnd16)++< expValue * 65535.0f;
-					else						acceptTheProposal=*(RND.rnd32)++< expValue * 4.294967296e+09;
+					if     (delta_lik > -0.5f)    acceptTheProposal=*(RND.rnd08)++< expValue * 255.0f;
+					else if(delta_lik > -5.0f  )  acceptTheProposal=*(RND.rnd16)++< expValue * 65535.0f;
+					else                          acceptTheProposal=*(RND.rnd32)++< expValue * 4.294967296e+09;					 
 				}
-				#ifdef __DEBUG__
-					if (basisIdx==0)++(flagS[NEW.jumpType]);
-					else++(flagT[NEW.jumpType]);
+				#if DEBUG_MODE==1
+                     if (basisIdx==0)++(flagS[NEW.jumpType]);
+                     else++(flagT[NEW.jumpType]);
+                     MEM.verify_header(&MEM);
 				#endif
 				if(acceptTheProposal)
 				{
-					#ifdef __DEBUG__
+					#if DEBUG_MODE==1
 						if (basisIdx==0)++(accS[NEW.jumpType]);
 						else++(accT[NEW.jumpType]);
 					#endif
 					if (yInfo.nMissing > 0 && Knewterm > 0 )  
 						f32_mat_multirows_set_by_submat(Xnewterm,Npad,Knewterm,Xt_zeroBackup,yInfo.rowsMissing,yInfo.nMissing);
-					if (NewCol.k2_old !=KOLD && NewCol.k2_new !=NewCol.k2_old)
-						shift_lastcols_within_matrix(Xt_mars,Npad,NewCol.k2_old+1,KOLD,NewCol.k2_new+1);
+					if (NEW.newcols.k2_old !=KOLD && NEW.newcols.k2_new !=NEW.newcols.k2_old)
+						shift_lastcols_within_matrix(Xt_mars,Npad,NEW.newcols.k2_old+1,KOLD,NEW.newcols.k2_new+1);
 					if (Knewterm !=0)
-						SCPY(Knewterm*Npad,Xnewterm,Xt_mars+(NewCol.k1-1) * Npad);
-					if (basis->type==OUTLIERID) {
-						basis->UpdateGoodVec_KnotList(basis,&NEW,Npad16);
-					} else {
-						basis->KNOT[-1]=basis->KNOT[INDEX_FakeStart];	basis->KNOT[basis->nKnot]=basis->KNOT[INDEX_FakeEnd];
-						basis->UpdateGoodVec_KnotList(basis,&NEW,Npad16);
-						basis->KNOT[-1]=1; 	                      	basis->KNOT[basis->nKnot]=N+1L;
-					}					
+						SCPY(Knewterm*Npad,Xnewterm,Xt_mars+(NEW.newcols.k1-1) * Npad);
+					 (basis->KNOT[-1]=basis->KNOT[INDEX_FakeStart],basis->KNOT[basis->nKnot]=basis->KNOT[INDEX_FakeEnd]);
+					 basis->UpdateGoodVec_KnotList(basis,&NEW,Npad16);
+					 (basis->KNOT[-1]=1,basis->KNOT[basis->nKnot]=N+1L);
 					basis->CalcBasisKsKeK_TermType(basis);
 					UpdateBasisKbase(MODEL.b,MODEL.NUMBASIS,basis-MODEL.b);
-					{
-						#define Exchange(x,y)   {void * _restrict tmp; tmp=MODEL.x;  MODEL.x=MODEL.y; MODEL.y=tmp;}
-						Exchange(curr.XtX,prop.XtX);
-						Exchange(curr.XtY,prop.XtY);
-						Exchange(curr.beta_mean,prop.beta_mean);
-						Exchange(curr.cholXtX,prop.cholXtX);
-						Exchange(curr.precXtXDiag,prop.precXtXDiag);        
-						Exchange(curr.nTermsPerPrecGrp,prop.nTermsPerPrecGrp);   
-						Exchange(curr.alpha2Q_star,prop.alpha2Q_star);           
+					{   
+						#define Exchange(x)          {VOIDPTR tmp=MODEL.curr.x;  MODEL.curr.x=MODEL.prop.x; MODEL.prop.x=tmp;}
+						#define Exchange2(x,y)       Exchange(x) Exchange(y) 
+						#define Exchange4(x,y,z,k)   Exchange(x) Exchange(y) Exchange(z) Exchange(k)
+						Exchange4(XtX,XtY,cholXtX,beta_mean);   
+						Exchange2(precXtXDiag,nTermsPerPrecGrp);  
+						Exchange(alpha2Q_star);                    
 						MODEL.curr.marg_lik=MODEL.prop.marg_lik;
-						MODEL.curr.K=MODEL.prop.K;  
-						#undef Exchange											
+						MODEL.curr.K=MODEL.prop.K;     
 					}
-					#ifdef __DEBUG__	
+					#if  DEBUG_MODE==1 
 					if (q==1) {
 					}
 					else {
-						MR_EvaluateModel(&MODEL.prop,MODEL.b,Xdebug,N,MODEL.NUMBASIS,&yInfo,&hyperPar,MODEL.precVec,&stream);
+						BEAST2_EvaluateModel(&MODEL.prop,MODEL.b,Xdebug,N,MODEL.NUMBASIS,&yInfo,&hyperPar,MODEL.precState.precVec,&stream);
 					}
 					#endif
 				} 
-				U08 bResampleParameter=(ite >=100)         && (ite%20==0)     ; 
-				U08 bStoreCurrentSample=(ite > MCMC_BURNIN) && (ite%MCMC_THINNING==0);
-				if (bResampleParameter||bStoreCurrentSample)	{		
-					if (q==1) {
-						F32 sig2=(*RND.rndgamma++) * 1.f/MODEL.curr.alpha2Q_star[0];
-						sig2=1.0f/sig2;					 
+				const int bResampleParameter=(ite >=100)         && (ite%20==0)     ;      
+				const int bStoreCurrentSample=(ite > MCMC_BURNIN) && (ite%MCMC_THINNING==0);
+				if (q==1) {
+					if (bResampleParameter||bStoreCurrentSample)	{	 
+						F32 sig2_inv=(*RND.rndgamma++) * 1.f/MODEL.curr.alpha2Q_star[0];
+						F32 sig2=1.0f/sig2_inv;
 						MODEL.sig2[0]=sig2 > MIN_SIG2_VALUE ? sig2 : MODEL.sig2[0];
-					}	else {
-						F32PTR MEMBUF=Xnewterm;
-						local_pcg_invwishart_upper( &stream,MODEL.sig2,MODEL.sig2+q*q,MEMBUF,q,
-							                        MODEL.curr.alpha2Q_star,hyperPar.alpha_1+yInfo.n+q - 1);					
+					}	 
+					if (bResampleParameter||(bStoreCurrentSample && extra.useRndBeta)) {
+							I32 K=MODEL.curr.K;
+							r_vsRngGaussian(VSL_RNG_METHOD_GAUSSIAN_ICDF,stream,K,MODEL.beta,0,1);
+							solve_U_as_U_invdiag(MODEL.curr.cholXtX,MODEL.beta,K,K);
+							r_ippsMulC_32f_I(fastsqrt(MODEL.sig2[0]),MODEL.beta,K);
+							r_ippsAdd_32f_I(MODEL.curr.beta_mean,MODEL.beta,K);
 					}
-				}
-				if (bResampleParameter||(bStoreCurrentSample && extra.useMeanOrRndBeta)) {
-					if (q==1) {
-						I32 K=MODEL.curr.K;
-						r_vsRngGaussian(VSL_RNG_METHOD_GAUSSIAN_ICDF,stream,K,MODEL.beta,0,1);
-						solve_U_as_U_invdiag(MODEL.curr.cholXtX,MODEL.beta,K,K);
-						r_ippsMulC_32f_I(fastsqrt(MODEL.sig2[0]),MODEL.beta,K);
-						r_ippsAdd_32f_I(MODEL.curr.beta_mean,MODEL.beta,K);
-					} else {
-						F32PTR MEMBUF=Xnewterm;
-						I32    K=MODEL.curr.K;
+					if (bResampleParameter ) {
+						I32 ntries=0;
+						do {
+							if (ntries++==0)	precFunc.ResamplePrecValues(&MODEL,&hyperPar,&stream);
+							else				precFunc.IncreasePrecValues(&MODEL);
+							precFunc.SetPrecXtXDiag(MODEL.curr.precXtXDiag,MODEL.b,MODEL.NUMBASIS,&MODEL.precState);
+							precFunc.chol_addCol(MODEL.curr.XtX,MODEL.curr.cholXtX,MODEL.curr.precXtXDiag,MODEL.curr.K,1L,MODEL.curr.K);
+							precFunc.ComputeMargLik(&MODEL.curr,&MODEL.precState,&yInfo,&hyperPar);
+						} while (IsNaN(MODEL.curr.marg_lik) && ntries < 20);
+						if (IsNaN(MODEL.curr.marg_lik)) {
+                             #if !(defined(R_RELEASE)||defined(M_RELEASE)||defined(P_RELEASE)) 
+							 r_printf("skip3|prec: %.4f|marg_lik_cur: %.4f \n",MODEL.precState.precVec[0],MODEL.curr.marg_lik);
+                             #endif
+							skipCurrentPixel=3;
+							break;
+						}
+					}
+				} 
+				if (q !=1) {    
+					F32PTR MEMBUF=Xnewterm;
+					I32    K=MODEL.curr.K;
+					if (bResampleParameter||bStoreCurrentSample) {						
+						local_pcg_invwishart_upper(&stream,MODEL.sig2,MODEL.sig2+q * q,MEMBUF,q,MODEL.curr.alpha2Q_star,hyperPar.alpha_1+yInfo.n+q - 1);
+					}
+					if (bResampleParameter||(bStoreCurrentSample && extra.useRndBeta)) {
 						r_vsRngGaussian(VSL_RNG_METHOD_GAUSSIAN_ICDF,stream,K * q,MEMBUF,0.,1.);
 						r_cblas_sgemm(CblasColMajor,CblasNoTrans,CblasNoTrans,K,q,q,1.0,MEMBUF,K,MODEL.sig2,q,0.f,MODEL.beta,K);
 						solve_U_as_U_invdiag_multicols(MODEL.curr.cholXtX,MODEL.beta,K,K,q);
 						r_ippsAdd_32f_I(MODEL.curr.beta_mean,MODEL.beta,K * q);
 					}
-				}
-				if (bResampleParameter && q==1) 
-				{
-					I32 ntries=0;
-					do {
-						if (ntries++==0)	precFunc.ResamplePrecValues(&MODEL,&hyperPar,&stream);							
-						else				precFunc.IncreasePrecValues(&MODEL);											
-						precFunc.GetXtXPrecDiag( &MODEL);
-						precFunc.chol_addCol(    MODEL.curr.XtX,MODEL.curr.cholXtX,MODEL.curr.precXtXDiag,MODEL.curr.K,1L,MODEL.curr.K);		
-						precFunc.ComputeMargLik( &MODEL.curr,&MODEL,&yInfo,&hyperPar);
-					} while (  IsNaN(MODEL.curr.marg_lik) && ntries < 20 );
-					if ( IsNaN(MODEL.curr.marg_lik) ) {
-						#if !(defined(R_RELEASE)||defined(M_RELEASE)) 
-						r_printf("skip3|prec: %.4f|marg_lik_cur: %.4f \n",MODEL.precVec[0],MODEL.curr.marg_lik);
-						#endif
-						skipCurrentPixel=3;
-						break;
-					} 
-				}
-				if (bResampleParameter && q>1) 
-				{
-					F32PTR MEMBUF=Xnewterm;					
-					I32    K=MODEL.curr.K;
-					I32 ntries=0;
-					do {
-						if (ntries++==0) {
-							r_cblas_sgemm(CblasColMajor,CblasNoTrans,CblasNoTrans,K,q,q,1.0,MODEL.beta,K,MODEL.sig2+q*q,q,0.f,MEMBUF,K);
-							F32 sumq=DOT(K * q,MEMBUF,MEMBUF);
-							r_vsRngGamma(VSL_RNG_METHOD_GAMMA_GNORM_ACCURATE,stream,1L,MODEL.precVec,(hyperPar.del_1+K * q * 0.5f),0.f,1.f);
-							MODEL.precVec[0]=MODEL.precVec[0]/(hyperPar.del_2+0.5f * sumq);
-							MODEL.logPrecVec[0]=logf(MODEL.precVec[0]);
-						} else {
-							precFunc.IncreasePrecValues(&MODEL);
+					if (bResampleParameter )		{						
+						I32 ntries=0;
+						do {
+							if (ntries++==0) {
+								r_cblas_sgemm(CblasColMajor,CblasNoTrans,CblasNoTrans,K,q,q,1.0,MODEL.beta,K,MODEL.sig2+q * q,q,0.f,MEMBUF,K);
+								F32 sumq=DOT(K * q,MEMBUF,MEMBUF);
+								r_vsRngGamma(VSL_RNG_METHOD_GAMMA_GNORM_ACCURATE,stream,1L,MODEL.precState.precVec,(hyperPar.del_1+K * q * 0.5f),0.f,1.f);
+								MODEL.precState.precVec[0]=MODEL.precState.precVec[0]/(hyperPar.del_2+0.5f * sumq);
+								MODEL.precState.logPrecVec[0]=logf(MODEL.precState.precVec[0]);
+							}	else {
+								precFunc.IncreasePrecValues(&MODEL);
+							}
+							precFunc.SetPrecXtXDiag(MODEL.curr.precXtXDiag,MODEL.b,MODEL.NUMBASIS,&MODEL.precState);
+							precFunc.chol_addCol(MODEL.curr.XtX,MODEL.curr.cholXtX,MODEL.curr.precXtXDiag,K,1,K);
+							precFunc.ComputeMargLik(&MODEL.curr,&MODEL.precState,&yInfo,&hyperPar);
+						} while (IsNaN(MODEL.curr.marg_lik) && ntries < 20);
+						if (IsNaN(MODEL.curr.marg_lik)) {
+                            #if !(defined(R_RELEASE)||defined(M_RELEASE)||defined(P_RELEASE))
+							 r_printf("skip4|prec: %.4f|marg_lik_cur: %.4f \n",MODEL.precState.precVec[0],MODEL.curr.marg_lik);
+                            #endif
+							skipCurrentPixel=3;
+							break;
 						}
-						precFunc.GetXtXPrecDiag(&MODEL);
-						precFunc.chol_addCol(MODEL.curr.XtX,MODEL.curr.cholXtX,MODEL.curr.precXtXDiag,K,1,K);
-						precFunc.ComputeMargLik(&MODEL.curr,&MODEL,&yInfo,&hyperPar);
-					} while (IsNaN(MODEL.curr.marg_lik) && ntries < 20);
-					if ( IsNaN(MODEL.curr.marg_lik) ) {
-						#if !(defined(R_RELEASE)||defined(M_RELEASE)) 
-							r_printf("skip4|prec: %.4f|marg_lik_cur: %.4f \n",MODEL.precVec[0],MODEL.curr.marg_lik);
-						#endif
-						skipCurrentPixel=3;
-						break;
-					}  
-				}
+					} 
+				} 
 				if (!bStoreCurrentSample) continue;
+				#if DEBUG_MODE==1
+					MEM.verify_header(&MEM);
+				#endif
 				sample++;
 				*resultChain.marg_lik+=MODEL.curr.marg_lik;
 				if (q==1) {
@@ -442,22 +398,21 @@ int beast2_main_corev4_mthrd(void* dummy)
 					r_cblas_sgemm(CblasColMajor,CblasTrans,CblasNoTrans,q,q,q,1.f,MODEL.sig2,q,MODEL.sig2,q,0.f,MEMBUF,q);
 					r_ippsAdd_32f_I(MEMBUF,resultChain.sig2,q*q);
 				}
-				F32PTR BETA=(extra.useMeanOrRndBeta==0) ? MODEL.curr.beta_mean : MODEL.beta;
+				const F32PTR BETA=(extra.useRndBeta==0) ? MODEL.curr.beta_mean : MODEL.beta;
 				{
-					F32PTR MEMBUF1=Xnewterm;					
+					 F32PTR MEMBUF1=Xnewterm;
 					for (I32 i=0; i < MODEL.NUMBASIS;++i) 
 					{
-						BEAST2_BASIS_PTR  basis=MODEL.b+i;
-						CORESULT        * result=coreResults+i;
-						I32        nKnot=basis->nKnot;
-						TKNOT_PTR  KNOT=basis->KNOT;
+						const BEAST2_BASIS_PTR  basis=MODEL.b+i;
+						const CORESULT        * result=coreResults+i;
+						const I32        nKnot=basis->nKnot;
+						const TKNOT_PTR  KNOT=basis->KNOT;
 						result->xNProb[nKnot]+=1L;
-						for (I32 i=0; i < nKnot; i++) result->xProb[ KNOT[i]-1 ]+=1L;
+						for (I32 j=0; j < nKnot; j++) result->xProb[ KNOT[j]-1 ]+=1L;
 						if (result->xorder !=NULL) {
-							TORDER_PTR  ORDER=basis->ORDER;
-							for (I32 i=0; i <=nKnot;++i) {
-								I32 r1=KNOT[i-1],r2=KNOT[i]-1;
-								r_ippsAddC_32s_ISfs(ORDER[i],result->xorder+r1 - 1,r2 - r1+1,0);
+							for (I32 j=0; j <=nKnot;++j) {
+								I32 r1=KNOT[j-1],r2=KNOT[j]-1;
+								r_ippsAddC_32s_ISfs(basis->ORDER[j],result->xorder+r1 - 1,r2 - r1+1,0);
 							}
 						}
 						if (q==1) {
@@ -469,7 +424,7 @@ int beast2_main_corev4_mthrd(void* dummy)
 							F32PTR  beta=BETA+basis->Kbase;
 							I32     K=basis->K;
 							r_cblas_sgemm(CblasColMajor,CblasNoTrans,CblasNoTrans,N,q,K,1.0f,
-									     	      X,Npad,beta,MODEL.curr.K,0.f,MEMBUF1,N);
+                                                                       X,Npad,beta,MODEL.curr.K,0.f,MEMBUF1,N);
 							f32_add_v_v2_vec_inplace(MEMBUF1,result->x,result->xSD,N*q);
 							MEMBUF1+=Npad*q;
 						}
@@ -477,53 +432,48 @@ int beast2_main_corev4_mthrd(void* dummy)
 				}
 				if(extra.computeSeasonAmp) 
 				{
-					F32PTR           MEMBUF1=Xnewterm+3*Npad;
-					F32PTR           MEMBUF2=MODEL.prop.beta_mean; 
-					BEAST2_BASIS_PTR basis=&MODEL.b[MODEL.sid];
-					I32             knotNum=basis->nKnot;
-					TKNOT_PTR       knotList=basis->KNOT;
-					F32PTR       beta=BETA;
-					TORDER_PTR   orderList=basis->ORDER;
-					F32PTR       SEASON_SQR_CSUM=basis->bConst.season.SQR_CSUM+1L;  
-					F32PTR       SEASON_SCALE=basis->bConst.season.SCALE_FACTOR;
+					const F32PTR           MEMBUF1=Xnewterm+3*Npad;
+					const BEAST2_BASIS_PTR basis=&MODEL.b[MODEL.sid];
+					const I32              knotNum=basis->nKnot;
+					const TKNOT_PTR        knotList=basis->KNOT;
+					F32PTR             beta=BETA;
+					const TORDER_PTR   orderList=basis->ORDER;
+					const F32PTR       SEASON_SQR_CSUM=basis->bConst.season.SQR_CSUM+1L;  
+					const F32PTR       SEASON_SCALE=basis->bConst.season.SCALE_FACTOR;
 					if (SEASON_BTYPE==0) {
 						for (I32 i=0; i <=knotNum; i++) {
-							I32		r1=knotList[i - 1];
-							I32		r2=knotList[i] - 1;
-							F32PTR seasonSqrCsum=SEASON_SQR_CSUM;
-							I32    order2=orderList[i] * 2L;
-							F32    amp=0;					 
+							const I32	r1=knotList[i - 1];
+							const I32	r2=knotList[i] - 1;							
+							const I32   order2=orderList[i] * 2L;
+							F32PTR seasonSqrCsum=SEASON_SQR_CSUM;							
+							F32    amp=0;
 							for (I32 j=0; j < order2; j++) {
 								F32 scalingFactor=N/(seasonSqrCsum[r2 - 1] - seasonSqrCsum[(r1 - 1) - 1]);
 								F32 beta0=beta[j]* SEASON_SCALE[j];
 								amp=amp+(beta0 * beta0) * scalingFactor;
 								seasonSqrCsum+=(N+1LL);
 							}			 
-							I32    segLength=r2 - r1+1L;
-							r_ippsSet_32f(amp,MEMBUF1+r1 - 1,segLength);
+							r_ippsSet_32f(amp,MEMBUF1+r1 - 1,r2 - r1+1L);
 							beta+=order2;
 						}
 					} else {
 						for (I32 i=0; i <=knotNum; i++) {
-							I32 r1=knotList[i - 1];
-							I32 r2=knotList[i] - 1;
-							F32PTR seasonSqrCsum=SEASON_SQR_CSUM;
-							I32    order2=orderList[i] * 2L;
+							const I32	r1=knotList[i - 1];
+							const I32	r2=knotList[i] - 1;							
+							const I32   order2=orderList[i] * 2L;			
 							F32    amp=0;		 
-							for (I32 j=0; j < order2; j++) {
+							for (I32 j=0; j < order2;++j) {
 								F32    beta0=beta[j] * SEASON_SCALE[j];
 								amp+=beta0 * beta0;
 							}					 
-							I32    segLength=r2 - r1+1L;
-							r_ippsSet_32f(amp,MEMBUF1+r1 - 1,segLength);
+							r_ippsSet_32f(amp,MEMBUF1+r1 - 1,r2 - r1+1L);
 							beta+=order2;
 						}
 					}
 					r_ippsAdd_32f_I(MEMBUF1,resultChain.samp,N);
 					r_ippsMul_32f_I(MEMBUF1,MEMBUF1,N);
 					r_ippsAdd_32f_I(MEMBUF1,resultChain.sampSD,N); 
-					if (extra.tallyPosNegSeasonJump)
-					{
+					if (extra.tallyPosNegSeasonJump) { 			 
 						I32  posKnotNum=0;
 						for (I32 i=0; i < knotNum; i++) { 
 							I64 knot=knotList[i];
@@ -538,11 +488,11 @@ int beast2_main_corev4_mthrd(void* dummy)
 				}
 				if(extra.computeTrendSlope)
 				{
-					BEAST2_BASIS_PTR basis=&MODEL.b[MODEL.tid];
-					I32             knotNum=basis->nKnot;
-					TKNOT_PTR       knotList=basis->KNOT;
-					F32PTR TREND=Xnewterm+Npad * MODEL.tid;     
-					F32PTR SLP=Xnewterm+3 * Npad;				
+					const BEAST2_BASIS_PTR basis=&MODEL.b[MODEL.tid];
+					const I32              knotNum=basis->nKnot;
+					const TKNOT_PTR        knotList=basis->KNOT;
+					const F32PTR TREND=Xnewterm+Npad * MODEL.tid;      
+					const F32PTR SLP=Xnewterm+Npad * MODEL.NUMBASIS; 
 					f32_diff_back(TREND,SLP,N);
 					f32_add_v_v2_vec_inplace(SLP,resultChain.tslp,resultChain.tslpSD,N); 
 					i32_increment_vec2_bycond_inplace(resultChain.tslpSgnPosPr,resultChain.tslpSgnZeroPr,SLP,N); 
@@ -558,7 +508,7 @@ int beast2_main_corev4_mthrd(void* dummy)
 						resultChain.tpos_ncpPr[posKnotNum]+=1L;
 						resultChain.tneg_ncpPr[knotNum - posKnotNum]+=1L;
 					}
-					if (extra.tallyIncDecTrendJump ){
+					if (extra.tallyIncDecTrendJump ){			 						
 						I32  incKnotNum=0;
 						for (I32 i=0; i < knotNum; i++) {  
 							I64 knot=knotList[i];
@@ -571,11 +521,10 @@ int beast2_main_corev4_mthrd(void* dummy)
 						resultChain.tdec_ncpPr[knotNum-incKnotNum]+=1L;
 					}
 				}
-				if(extra.tallyPosNegOutliers)
-				{
-					BEAST2_BASIS_PTR basis=&MODEL.b[MODEL.oid];
-					rI32             knotNum=basis->nKnot;
-					rTKNOT_PTR       knotList=basis->KNOT;
+				if(extra.tallyPosNegOutliers) {
+					const BEAST2_BASIS_PTR basis=&MODEL.b[MODEL.oid];
+					const I32              knotNum=basis->nKnot;
+					const TKNOT_PTR        knotList=basis->KNOT;
 					const F32PTR OUTLIIER=Xnewterm+Npad* MODEL.oid;
 					I32  posKnotNum=0;
 					for (I32 i=0; i < knotNum; i++) {  
@@ -589,9 +538,8 @@ int beast2_main_corev4_mthrd(void* dummy)
 					resultChain.oneg_ncpPr[knotNum - posKnotNum]+=1L;								
 				}
 				if (extra.computeCredible)	{ 	
-					if (extra.fastCIComputation &&  !(*RND.rnd16++< ciParam.subsampleFraction_x_INT16MAX)  ){
-					} else {
-						for (int i=0; i < MODEL.NUMBASIS; i++) 
+					if ( !extra.fastCIComputation||*RND.rnd16++< ciParam.subsampleFraction_x_INT16MAX  ) {
+						for (int i=0; i <  NumCIVars; i++) 
 							InsertNewRowToUpdateCI(&ciParam,&ci[i]);						
 					}					
 				} 
@@ -640,7 +588,7 @@ int beast2_main_corev4_mthrd(void* dummy)
 							F32 offset=0.0f;
 							f32_sx_sxx_to_avgstd_inplace(resultChain.tY+i * N,resultChain.tSD+i * N,sample,yInfo.sd[i],yInfo.mean[i],N);
 						}
-						if (extra.computeTrendOrder) 	i32_to_f32_scaleby_inplace(resultChain.torder,N,inv_sample);						
+						if (extra.computeTrendOrder) 	i32_to_f32_scaleby_inplace(resultChain.torder,N,inv_sample);	
 						if (extra.computeTrendSlope) {
 							for (int i=0; i < q; i++) {
 								f32_sx_sxx_to_avgstd_inplace(resultChain.tslp+i*N,resultChain.tslpSD+i*N,sample,yInfo.sd[i]/opt->io.meta.deltaTime,0,N);
@@ -652,6 +600,12 @@ int beast2_main_corev4_mthrd(void* dummy)
 							for (int i=0; i < q; i++) {
 								f32_scale_inplace(yInfo.sd[i],yInfo.mean[i],resultChain.tCI+N * i,N);
 								f32_scale_inplace(yInfo.sd[i],yInfo.mean[i],resultChain.tCI+N*q+N * i,N);
+							}	
+							if (extra.computeTrendSlope) {
+								for (int i=0; i < q; i++) {
+									f32_mul_val_inplace(yInfo.sd[i],resultChain.tslpCI+N * i,N);
+									f32_mul_val_inplace(yInfo.sd[i],resultChain.tslpCI+N * q+N * i,N);
+								}						
 							}							
 						}
 				}
@@ -738,6 +692,7 @@ int beast2_main_corev4_mthrd(void* dummy)
 					if (extra.computeTrendOrder)   _N(torder);
 					if (extra.computeTrendSlope)   _N(tslp),_N(tslpSD),_N(tslpSgnPosPr),_N(tslpSgnZeroPr);
 					if (extra.computeCredible)     _2Nq(tCI);
+					if (extra.computeCredible && extra.computeTrendSlope ) _2Nq(tslpCI);
 				}
 				if (MODEL.oid >=0) {
 					_1(oncp); _okn_1(oncpPr);	_N(ocpOccPr); _Nq(oY); _Nq(oSD);
@@ -775,7 +730,7 @@ int beast2_main_corev4_mthrd(void* dummy)
 			    #undef _okn_1
 			}
 			if (skipCurrentPixel) {
-				break;
+			     break;
 			}
 		}
 		__END_IF_NOT_SKIP_TIMESESIRIES__  
@@ -1078,8 +1033,8 @@ int beast2_main_corev4_mthrd(void* dummy)
 		}
 		BEAST2_WriteOutput(opt,&result,pixelIndex);
 		pthread_mutex_lock(&mutex);
-		NUM_OF_PROCESSED_GOOD_PIXELS+=!skipCurrentPixel;  
-		NUM_OF_PROCESSED_PIXELS++;							
+		NUM_OF_PROCESSED_GOOD_PIXELS+=!skipCurrentPixel;              
+		NUM_OF_PROCESSED_PIXELS++;					
 		pthread_mutex_unlock(&mutex);
 		F32 elaspedTime=GetElaspedTimeFromBreakPoint();
 		if (NUM_OF_PROCESSED_GOOD_PIXELS > 0 && NUM_PIXELS > 1 && (pixelIndex%50==0||elaspedTime > 1))  {
@@ -1089,15 +1044,15 @@ int beast2_main_corev4_mthrd(void* dummy)
 				SetBreakPointForStartedTimer();
 			}
 		}
-		if (IDE_USER_INTERRUPT)		{
-			break;
-		}
-		#ifdef __DEBUG__
+		#if DEBUG_MODE==1
 		r_printf("TREND: birth%4d/%-5d|death%4d/%-5d|merge%4d/%-5d|move%4d/%-5d|chorder%4d/%-5d\n", 
 			      accT[0],flagT[0],accT[1],flagT[1],accT[2],flagT[2],accT[3],flagT[3],accT[4],flagT[4]);
 		r_printf("SEASN: birth%4d/%-5d|death%4d/%-5d|merge%4d/%-5d|move%4d/%-5d|chorder%4d/%-5d\n",
 			      accS[0],flagS[0],accS[1],flagS[1],accS[2],flagS[2],accS[3],flagS[3],accS[4],flagS[4]);
 		#endif
+		if (IDE_USER_INTERRUPT)		{
+			break;
+		}
 	} 
 	r_vslDeleteStream(&stream);
 	MEM.free_all(&MEM);
