@@ -6,6 +6,30 @@
 #include "abc_date.h"
 #include "abc_mem.h"
 #include "beastv2_io.h"
+static void* __BEAST2_Output_AllocMEM_MCMCSamples(A(OPTIONS_PTR)  opt) {
+	const BEAST2_IO_PTR      io=&opt->io;
+	const BEAST2_RESULT_PTR  mat=io->out.result;
+	DATA_TYPE   dtype=io->out.dtype; 
+	const int   N=io->N;
+	I08 hasSeasonCmpnt=opt->prior.basisType[0]==SEASONID||opt->prior.basisType[0]==DUMMYID||opt->prior.basisType[0]==SVDID;
+	I08 hasDummyCmpnt=opt->prior.basisType[0]==DUMMYID;
+	I08 hasTrendCmpnt=1;
+	I08 hasOutlierCmpnt=opt->prior.basisType[opt->prior.numBasis - 1]==OUTLIERID;
+	I32 nsamples=opt->mcmc.samples;
+	I32 nchains=opt->mcmc.chainNumber;
+	I32 q=io->q;
+	FIELD_ITEM  fieldList[]={
+	    { "season",dtype,3+(q>1),{ N,nsamples,nchains,q},&mat->smcmc },
+		{ "trend",dtype,3+(q > 1),{ N,nsamples,nchains,q},&mat->tmcmc },
+		{ "outlier",dtype,3+(q > 1),{ N,nsamples,nchains,q},&mat->omcmc }
+	};
+	I32 nfields=sizeof(fieldList)/sizeof(FIELD_ITEM);
+	if (!hasSeasonCmpnt)    fieldList[0].ptr=NULL;
+	if (!hasOutlierCmpnt)   fieldList[2].ptr=NULL;
+	VOID_PTR  out=PROTECT(CreateStructVar(fieldList,nfields));
+					 UNPROTECT(1L);
+	return out;
+}
 static void __RemoveFieldsGivenFlags_Trend(A(OPTIONS_PTR)  opt,FIELD_ITEM * fieldList,int nfields) {
 	I08 hasSeasonCmpnt=opt->prior.basisType[0]==SEASONID||opt->prior.basisType[0]==DUMMYID||opt->prior.basisType[0]==SVDID;
 	I08 hasOutlierCmpnt=opt->prior.basisType[opt->prior.numBasis - 1]==OUTLIERID;
@@ -21,6 +45,8 @@ static void __RemoveFieldsGivenFlags_Trend(A(OPTIONS_PTR)  opt,FIELD_ITEM * fiel
 		RemoveField(fieldList,nfields,"slpSD"),mat->tslpSD=NULL,
 		RemoveField(fieldList,nfields,"slpSgnPosPr"),mat->tslpSgnPosPr=NULL,
 	    RemoveField(fieldList,nfields,"slpSgnZeroPr"),mat->tslpSgnPosPr=NULL;	
+	if (!flag->computeTrendSlope||!flag->computeCredible)
+		RemoveField(fieldList,nfields,"slpCI"),mat->tslpCI=NULL;
 	if (!flag->computeTrendChngpt)
 		RemoveField(fieldList,nfields,"cp"),mat->tcp=NULL,
 		RemoveField(fieldList,nfields,"cpPr"),mat->tcpPr=NULL,
@@ -247,7 +273,9 @@ static void* __BEAST2_Output_AllocMEM_Trend(A(OPTIONS_PTR)  opt) {
 			_1(cpCI,mxKnotNum,2),
 			_q2(Y,SD,N),
 			_q(CI,N,2),
-			 _q4(slp,slpSD,slpSgnPosPr,slpSgnZeroPr,N),
+			 _q2(slp,slpSD,N),
+			 _q1(slpCI,N,2),
+			 _q2(slpSgnPosPr,slpSgnZeroPr,N),
 			 _q2(pos_ncp,neg_ncp,1),
 			 _q2(pos_ncpPr,neg_ncpPr,mxKnotNum+1),
 			 _q2(pos_cpOccPr,neg_cpOccPr,N),
@@ -442,10 +470,11 @@ void* BEAST2_Output_AllocMEM(A(OPTIONS_PTR)  opt)  {
 	I08 hasTrendCmpnt=1;
 	I08 hasOutlierCmpnt=opt->prior.basisType[opt->prior.numBasis - 1]==OUTLIERID;
 	I32       nprt=0;
-	VOID_PTR  trend=NULL,season=NULL,outlier=NULL;
+	VOID_PTR  trend=NULL,season=NULL,outlier=NULL,mcmc=NULL;
 	if (hasTrendCmpnt)   { trend=PROTECT(__BEAST2_Output_AllocMEM_Trend(opt));    nprt++; }
 	if (hasSeasonCmpnt)  { season=PROTECT(__BEAST2_Output_AllocMEM_Season(opt));   nprt++; }
 	if (hasOutlierCmpnt) { outlier=PROTECT(__BEAST2_Output_AllocMEM_Outlier(opt));  nprt++;}
+	if (opt->extra.dumpMCMCSamples) { mcmc=PROTECT(__BEAST2_Output_AllocMEM_MCMCSamples(opt));  nprt++; }
 	int   ROW,COL;
 	if (io->ndim==1||io->ndim==2) {
 		if (io->ndim==1||(io->ndim==2 && io->out.whichDimIsTime==1)) 			
@@ -496,7 +525,14 @@ void* BEAST2_Output_AllocMEM(A(OPTIONS_PTR)  opt)  {
 	VOID_PTR  out;
 	I32       nptr1=__MR_ExtendFieldsToMultiVaraiteTS(fieldList,nfields,io->q);  nprt+=nptr1;
 	if (ROW * COL==1) {
-		out=PROTECT(CreateStructVar(fieldList,nfields-3));                    nprt++;
+		if (!opt->extra.dumpMCMCSamples) {
+			out=PROTECT(CreateStructVar(fieldList,nfields - 3));          
+			nprt++;
+		} 	else {
+			fieldList[nfields - 3 ]=(FIELD_ITEM) { "mcmc",DATA_STRUCT,0,{0,},(void**)mcmc };
+			out=PROTECT(CreateStructVar(fieldList,nfields - 3+1)); 
+			nprt++;
+		}
 	} else {
 		out=PROTECT(CreateStructVar(fieldList,nfields));                      nprt++;		
 		whichDimIsTime[0]=io->out.whichDimIsTime;
@@ -511,6 +547,8 @@ void* BEAST2_Output_AllocMEM(A(OPTIONS_PTR)  opt)  {
 		for (int i=0; i < N; i++) {
 			mat->time[i]=FracYear_from_DateNum(io->meta.startTime+io->meta.deltaTime*i);
 		} 
+		io->meta.startTime=mat->time[0];
+		io->meta.deltaTime=mat->time[1]- mat->time[0];
 	}	else {
 		f32_seq(mat->time,io->meta.startTime,io->meta.deltaTime,N);
 	}    
@@ -602,28 +640,29 @@ void  BEAST2_Result_AllocMEM(A(RESULT_PTR)  result,A(OPTIONS_PTR)  opt,MemPointe
 		nodes[nid++]=(MemNode){ &result->oSD,sizeof(I32) * Nq,.align=64 };
 	}
 	if (opt->extra.computeCredible){
-		if (hasSeasonCmpnt)  nodes[nid++]=(MemNode){ &result->sCI,sizeof(F32) * Nq * 2,.align=4 };
-		if (hasTrendCmpnt)   nodes[nid++]=(MemNode){ &result->tCI,sizeof(F32) * Nq * 2,.align=4 }; 
-		if (hasOutlierCmpnt) nodes[nid++]=(MemNode){ &result->oCI,sizeof(F32) * Nq * 2,.align=4 }; 
+		if (hasSeasonCmpnt)                 nodes[nid++]=(MemNode){ &result->sCI,sizeof(F32) * Nq * 2,.align=4 };
+		if (hasTrendCmpnt)                  nodes[nid++]=(MemNode){ &result->tCI,sizeof(F32) * Nq * 2,.align=4 }; 
+		if (hasOutlierCmpnt)                nodes[nid++]=(MemNode){ &result->oCI,sizeof(F32) * Nq * 2,.align=4 }; 
+		if (opt->extra.computeTrendSlope)	nodes[nid++]=(MemNode){ &result->tslpCI,sizeof(F32) * Nq * 2,.align=4 };		 
 	}
-	if (opt->extra.computeSeasonChngpt && hasSeasonCmpnt) {
+	if (!!opt->extra.computeSeasonChngpt && hasSeasonCmpnt) {
 		nodes[nid++]=(MemNode){ &result->scp,sizeof(U32) * seasonMaxKnotNum,.align=4 };
 		nodes[nid++]=(MemNode){ &result->scpPr,sizeof(U32) * seasonMaxKnotNum,.align=4 };
 		nodes[nid++]=(MemNode){ &result->scpAbruptChange,sizeof(U32) * seasonMaxKnotNum * q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->scpCI,sizeof(U32) * seasonMaxKnotNum * 2,.align=4 };
   	}	
-	if (opt->extra.computeTrendChngpt && hasTrendCmpnt) {
+	if (!!opt->extra.computeTrendChngpt && hasTrendCmpnt) {
 		nodes[nid++]=(MemNode){ &result->tcp,sizeof(U32) * trendMaxKnotNum,.align=4 };
 		nodes[nid++]=(MemNode){ &result->tcpPr,sizeof(U32) * trendMaxKnotNum,.align=4 };
 		nodes[nid++]=(MemNode){ &result->tcpAbruptChange,sizeof(U32) * trendMaxKnotNum * q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->tcpCI,sizeof(U32) * trendMaxKnotNum * 2,.align=4 };
 	}
-	if (opt->extra.computeOutlierChngpt && hasOutlierCmpnt) {
+	if (!!opt->extra.computeOutlierChngpt && hasOutlierCmpnt) {
 		nodes[nid++]=(MemNode){ &result->ocp,sizeof(U32) * outlierMaxKnotNum,.align=4 };
 		nodes[nid++]=(MemNode){ &result->ocpPr,sizeof(U32) * outlierMaxKnotNum,.align=4 };
 		nodes[nid++]=(MemNode){ &result->ocpCI,sizeof(U32) * outlierMaxKnotNum * 2,.align=4 };
 	}
-	if (opt->extra.tallyPosNegSeasonJump && hasSeasonCmpnt) {
+	if (!!opt->extra.tallyPosNegSeasonJump && hasSeasonCmpnt) {
 		nodes[nid++]=(MemNode){ &result->spos_ncp,sizeof(F32) * 1*q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->sneg_ncp,sizeof(F32) * 1 * q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->spos_ncpPr,sizeof(I32) * (seasonMaxKnotNum+1) * q,.align=4 };
@@ -639,7 +678,7 @@ void  BEAST2_Result_AllocMEM(A(RESULT_PTR)  result,A(OPTIONS_PTR)  opt,MemPointe
 		nodes[nid++]=(MemNode){ &result->spos_cpCI,sizeof(U32) * seasonMaxKnotNum * 2*q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->sneg_cpCI,sizeof(U32) * seasonMaxKnotNum * 2*q,.align=4 };
 	}
-	if (opt->extra.tallyPosNegTrendJump && hasTrendCmpnt) {
+	if (!!opt->extra.tallyPosNegTrendJump && hasTrendCmpnt) {
 		nodes[nid++]=(MemNode){ &result->tpos_ncp,sizeof(F32) * 1*q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->tneg_ncp,sizeof(F32) * 1 * q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->tpos_ncpPr,sizeof(I32) * (trendMaxKnotNum+1) * q,.align=4 };
@@ -655,7 +694,7 @@ void  BEAST2_Result_AllocMEM(A(RESULT_PTR)  result,A(OPTIONS_PTR)  opt,MemPointe
 		nodes[nid++]=(MemNode){ &result->tpos_cpCI,sizeof(U32) * trendMaxKnotNum * 2*q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->tneg_cpCI,sizeof(U32) * trendMaxKnotNum * 2*q,.align=4 };
 	}
-	if (opt->extra.tallyIncDecTrendJump && hasTrendCmpnt) {
+	if (!!opt->extra.tallyIncDecTrendJump && hasTrendCmpnt) {
 		nodes[nid++]=(MemNode){ &result->tinc_ncp,sizeof(F32) * 1 * q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->tdec_ncp,sizeof(F32) * 1 * q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->tinc_ncpPr,sizeof(I32) * (trendMaxKnotNum+1) * q,.align=4 };
@@ -671,7 +710,7 @@ void  BEAST2_Result_AllocMEM(A(RESULT_PTR)  result,A(OPTIONS_PTR)  opt,MemPointe
 		nodes[nid++]=(MemNode){ &result->tinc_cpCI,sizeof(U32) * trendMaxKnotNum * 2 * q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->tdec_cpCI,sizeof(U32) * trendMaxKnotNum * 2 * q,.align=4 };
 	}
-	if (opt->extra.tallyPosNegOutliers && hasOutlierCmpnt) {
+	if (!!opt->extra.tallyPosNegOutliers && hasOutlierCmpnt) {
 		nodes[nid++]=(MemNode){ &result->opos_ncp,sizeof(F32) * 1 * q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->oneg_ncp,sizeof(F32) * 1 * q,.align=4 };
 		nodes[nid++]=(MemNode){ &result->opos_ncpPr,sizeof(I32) * (outlierMaxKnotNum+1) * q,.align=4 };
@@ -756,9 +795,10 @@ void  BEAST2_Result_FillMEM(A(RESULT_PTR)  result,A(OPTIONS_PTR)  opt,const F32
 		f32_fill_val(nan,result->oSD,Nq);
 	}
 	if (flag->computeCredible) 	{
-		if (hasSeasonCmpnt)  f32_fill_val(nan,result->sCI,2*Nq);
-		if (hasTrendCmpnt)   f32_fill_val(nan,result->tCI,2*Nq);
-		if (hasOutlierCmpnt) f32_fill_val(nan,result->oCI,2*Nq);
+		if ( hasSeasonCmpnt )               f32_fill_val(nan,result->sCI,2*Nq);
+		if ( hasTrendCmpnt  )               f32_fill_val(nan,result->tCI,2*Nq);
+		if ( hasOutlierCmpnt)               f32_fill_val(nan,result->oCI,2*Nq);
+		if ( opt->extra.computeTrendSlope)	f32_fill_val(nan,result->tslpCI,2*Nq);
 	}
 	if (flag->computeSeasonChngpt && hasSeasonCmpnt) 	{
 		f32_fill_val(nan,result->scp,seasonMaxKnotNum);
@@ -766,18 +806,18 @@ void  BEAST2_Result_FillMEM(A(RESULT_PTR)  result,A(OPTIONS_PTR)  opt,const F32
 		f32_fill_val(nan,result->scpAbruptChange,seasonMaxKnotNum*q);
 		f32_fill_val(nan,result->scpCI,2*seasonMaxKnotNum); 
 	}
-	if (flag->computeTrendChngpt && hasTrendCmpnt) {
+	if (!!flag->computeTrendChngpt && !!hasTrendCmpnt) {
 		f32_fill_val(nan,result->tcp,trendMaxKnotNum );
 		f32_fill_val(nan,result->tcpPr,trendMaxKnotNum );
 		f32_fill_val(nan,result->tcpAbruptChange,trendMaxKnotNum * q);
 		f32_fill_val(nan,result->tcpCI,2* trendMaxKnotNum );
 	}
-	if (flag->computeOutlierChngpt&& hasOutlierCmpnt) {
+	if (!!flag->computeOutlierChngpt && !!hasOutlierCmpnt) {
 		f32_fill_val(nan,result->ocp,outlierMaxKnotNum );
 		f32_fill_val(nan,result->ocpPr,outlierMaxKnotNum );
 		f32_fill_val(nan,result->ocpCI,2* outlierMaxKnotNum );
 	}
-	if (flag->tallyPosNegSeasonJump && hasSeasonCmpnt)  {
+	if (!!flag->tallyPosNegSeasonJump && !!hasSeasonCmpnt)  {
 		f32_fill_val(nan,result->spos_ncp,1*q);
 		f32_fill_val(nan,result->sneg_ncp,1 * q);
 		f32_fill_val(nan,result->spos_ncpPr,(seasonMaxKnotNum+1) * q);
@@ -793,7 +833,7 @@ void  BEAST2_Result_FillMEM(A(RESULT_PTR)  result,A(OPTIONS_PTR)  opt,const F32
 		f32_fill_val(nan,result->spos_cpCI,2*seasonMaxKnotNum * q);
 		f32_fill_val(nan,result->sneg_cpCI,2 * seasonMaxKnotNum * q);
 	}
-	if (flag->tallyPosNegTrendJump && hasTrendCmpnt) {
+	if (!!flag->tallyPosNegTrendJump && !!hasTrendCmpnt) {
 		f32_fill_val(nan,result->tpos_ncp,1 * q);
 		f32_fill_val(nan,result->tneg_ncp,1 * q);
 		f32_fill_val(nan,result->tpos_ncpPr,(trendMaxKnotNum+1) * q);
@@ -809,7 +849,7 @@ void  BEAST2_Result_FillMEM(A(RESULT_PTR)  result,A(OPTIONS_PTR)  opt,const F32
 		f32_fill_val(nan,result->tpos_cpCI,2* trendMaxKnotNum * q);
 		f32_fill_val(nan,result->tneg_cpCI,2 * trendMaxKnotNum * q);
 	}
-	if (flag->tallyIncDecTrendJump && hasTrendCmpnt) {
+	if (!!flag->tallyIncDecTrendJump && !!hasTrendCmpnt) {
 		f32_fill_val(nan,result->tinc_ncp,1 * q);
 		f32_fill_val(nan,result->tdec_ncp,1 * q);
 		f32_fill_val(nan,result->tinc_ncpPr,(trendMaxKnotNum+1) * q);
@@ -825,7 +865,7 @@ void  BEAST2_Result_FillMEM(A(RESULT_PTR)  result,A(OPTIONS_PTR)  opt,const F32
 		f32_fill_val(nan,result->tinc_cpCI,2* trendMaxKnotNum);
 		f32_fill_val(nan,result->tdec_cpCI,2 * trendMaxKnotNum); 
 	}
-	if (flag->tallyPosNegOutliers && hasOutlierCmpnt) {
+	if (!!flag->tallyPosNegOutliers && !!hasOutlierCmpnt) {
 		f32_fill_val(nan,result->opos_ncp,1 * q);
 		f32_fill_val(nan,result->oneg_ncp,1 * q);
 		f32_fill_val(nan,result->opos_ncpPr,(outlierMaxKnotNum+1)* q);
